// **********************************
// from basic code to apps: changing lines are delimited with
//                                             // +++++++ start
//                                             // +++++++ end
// (this warning comes from bp-ct-1.1, i.e the original application containing 
// this file)
// the file is modified here to use an ad hoc random distribution
// **********************************
//
// in apps we have to prepare a XXXInterface.m file for each type of agent:
// in the accompanying ct-hayek application of bp-ct this file corresponds
// to the files ConsumerInterface.m and ProducerInterface.m
//
// **********************************

// BPCTAgentAInterface.m                                        

// +++++++ start
#import "BPCTAgentAInterface.h"
// +++++++ end

@implementation BPCTAgentAInterface

- createEnd
{
  [super createEnd];

  return self;
}


  // all data are considered here in their external metrics; all the job
  // of tranforming the external metrics to the internal one (useful for
  // a neural network) and then to the external one again is done
  // automatically using the various matrixes, which are contained in objects
  // that respond to the ...asExternal... methods, which apply the internal
  // and external min-max intervals contained in the minmax matrix
                                                       
  // the following methods are strictly related to the meaning the problem
  // data, to which we are applying our neural network, have

  // so we have to modify them whenever we change the problem if we want
  // to display agent data or operate with them in a CT way

  // the following "get" methods link the external names (used by the model
  // builder as variables) to the inputs/outputs/targets of the neural
  // network, where they are identified by a position in the input, output,
  // target vectors

// +++++++ start

// input node 1 (position 0)
- (float) getMeanPrice5
          // meanPrice5 (5 stays for t-5, where t is 'today')
          // is the 1st input
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:0 C:0]+[minmax R:0 C:1])/2.; 
              // intermediate minmax value, as external

  // in this kind of CT use of bp-ct package, we have always only one
  // pattern in dataVarificationMatrix, so the loop explained below is 
  // eliminated, as well the 0.5 count

  // [in each "get" we access a different pattern, as in a loop,
  // [restarting from the first at the end (we use here the remainder of
  // [the integer quotient calculated below with the % or Mod operator);

  // [the count is made by 0.5; being k an int variable, we have to call
  // [twice the method to change pattern; this mechanism is requested by
  // [the use of this kind of messages in Observer.m, where EZGraph instances
  // [call twice this kind of method to graph and to send to file the value
  // [of each input, output or target

  k = 0;

  return [dataVerificationMatrix R: k C: 0 asExternalValueUsing: minmax
                                           withShift: 0];
  // withShift: 0 has the meaning of no shift, because minmax (row) position
  // matches perfectly dataVerificationMatrix (col) position
}

// input node 2 (position 1)
- (float) getMeanPrice4
          // meanPrice4 (4 stays for t-4, where t is 'today')
          // is the 2nd input
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:1 C:0]+[minmax R:1 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [dataVerificationMatrix R: k C: 1 asExternalValueUsing: minmax
                                           withShift: 0];
  // withShift: 0 has the meaning of no shift, because minmax (row) position
  // matches perfectly dataVerificationMatrix (col) position
}

// input node 3 (position 2)
- (float) getMeanPrice3
          // meanPrice3 (3 stays for t-3, where t is 'today')
          // is the 3th input
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:2 C:0]+[minmax R:2 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [dataVerificationMatrix R: k C: 2 asExternalValueUsing: minmax
                                           withShift: 0];
  // withShift: 0 has the meaning of no shift, because minmax (row) position
  // matches perfectly dataVerificationMatrix (col) position
}

// input node 4 (position 3)
- (float) getMeanPrice2
          // meanPrice2 (2 stays for t-2, where t is 'today')
          // is the 4th input
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:3 C:0]+[minmax R:3 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [dataVerificationMatrix R: k C: 3 asExternalValueUsing: minmax
                                           withShift: 0];
}

// input node 5 (position 4)
- (float) getMeanPrice1
          // meanPrice1 (1 stays for t-1, where t is 'today')
          // is the 5th input
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:4 C:0]+[minmax R:4 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [dataVerificationMatrix R: k C: 4 asExternalValueUsing: minmax
                                           withShift: 0];
}

// input node 6 (position 5)
- (float) getPreviousDayLiquidityQuantity
          // previousDayLiquitityQuantity, corresponding to 
          // liquidityQuantity_target of the previous day, 
          // is the 6th input
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:5 C:0]+[minmax R:5 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [dataVerificationMatrix R: k C: 5 asExternalValueUsing: minmax
                                           withShift: 0];
}

// input node 7 (position 6)
- (float) getPreviousDayShareQuantity
          // previousDayShareQuantity, corresponding to 
          // shareQuantity_target of the previous day, 
          // is the 8th input
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:6 C:0]+[minmax R:6 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [dataVerificationMatrix R: k C: 6 asExternalValueUsing: minmax
                                           withShift: 0];
}

// output node 1 (position 0 or 7)
- (float) getLiquidityQuantity_out
          // liquidityQuantity_out    is the output of the 1st effect
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:7 C:0]+[minmax R:7 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [outputVerificationMatrix R: k C: 0 asExternalValueUsing: minmax
                                              withShift: inputNodeNumber];
  // withShift: inputNodeNumber matches minmax (row)
  // position and outputVerificationMatrix (col) position
  // (we have to use it here in all "get" methods which are related to
  // outputs or targets)
}

// target node 1 (position 0 or 7)
- (float) getLiquidityQuantity_target
          // liquidityQuantity_target is the target of the 1st effect
{
  int k;

  // if not yet used
  if(used!=2) return ([minmax R:7 C:0]+[minmax R:7 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [targetVerificationMatrix R: k C: 0 asExternalValueUsing: minmax
                                              withShift: inputNodeNumber];
  // withShift: inputNodeNumber matches minmax (row)
  // position and targetVerificationMatrix (col) position
  // (we have to use it here in all "get" methods which are related to
  // outputs or targets)
}

// output node 2 (position 1 or 8)
- (float) getShareQuantity_out
          // shareQuantity_out        is the output of the 2nd effect
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:8 C:0]+[minmax R:8 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [outputVerificationMatrix R: k C: 1 asExternalValueUsing: minmax
                                              withShift: inputNodeNumber];
}

// target node 2 (position 1 or 8)
- (float) getShareQuantity_target
          // shareQuantity_target     is the target of the 2nd effect
{
  int k;

  // if not yet used
  if(used!=2) return ([minmax R:8 C:0]+[minmax R:8 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [targetVerificationMatrix R: k C: 1 asExternalValueUsing: minmax
                                              withShift: inputNodeNumber];
}

// output node 3 (position 2 or 9)
- (float) getBuySell_out
          // buySell_out              is the output of the 1st action
          // + buy; - sell
{
  int k;

  // if not yet used
  if(used==0) return ([minmax R:9 C:0]+[minmax R:9 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [outputVerificationMatrix R: k C: 2 asExternalValueUsing: minmax
                                              withShift: inputNodeNumber];
}

// target node 3 (position 2 or 9)
- (float) getBuySell_target
          // buySell_target           is the target of the 1st action
          // + buy; - sell
{
  int k;

  // if not yet used
  if(used!=2) return ([minmax R:9 C:0]+[minmax R:9 C:1])/2.; 
              // intermediate minmax value, as external


  k = 0;

  return [targetVerificationMatrix R: k C: 2 asExternalValueUsing: minmax
                                              withShift: inputNodeNumber];
}

// +++++++ end


- setCT_Inputs
{    

  // preserving previuously generated data, in CT scheme
  // when abs(patternNumberInTrainingSet)>1 ;
  // if abs(patternNumberInTraingingSet)=1 'shift' returns without effects
  [dataTrainingMatrix shift];

  // inputs:
  // we have to provide inputs (the same) both to the
  // dataVerificationMatrix (used in the no-learning acting phase in CT)
  // and to the dataTrainingMatrix (used in learning phase)

  // the learning phase uses one or more patterns and may be repeated in
  // each step (see the HowToUseBp-Ct.txt in the bp-ct package)
  // we store the data always in dataTrainingMatrix last row; the shift
  // order done above assures that the line is free and previous data are
  // preserved
  
// +++++++ start


  // meanPrice5 (5 stays for t-5, where t is 'today')
  // is the 1st input
  [dataVerificationMatrix R: 0 C: 0 setFrom:     [self getMeanPrice4]   
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 0 setFrom: [self getMeanPrice4] 
                          asExternalValueUsing: minmax withShift: 0];
  // meanPrice4 (4 stays for t-4, where t is 'today')
  // is the 2nd input
  [dataVerificationMatrix R: 0 C: 1 setFrom:     [self getMeanPrice3]   
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 1 setFrom: [self getMeanPrice3] 
                          asExternalValueUsing: minmax withShift: 0];
  // meanPrice3 (3 stays for t-3, where t is 'today')
  // is the 3th input
  [dataVerificationMatrix R: 0 C: 2 setFrom:     [self getMeanPrice2]   
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 2 setFrom: [self getMeanPrice2] 
                          asExternalValueUsing: minmax withShift: 0];
  // meanPrice2 (2 stays for t-2, where t is 'today')
  // is the 4th input
  [dataVerificationMatrix R: 0 C: 3 setFrom:     [self getMeanPrice1]   
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 3 setFrom: [self getMeanPrice1] 
                          asExternalValueUsing: minmax withShift: 0];
  // meanPrice1 (1 stays for t-1, where t is 'today')
  // is the 5th input
  [dataVerificationMatrix R: 0 C: 4 setFrom:     [oneBook getMeanPrice]   
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 4 setFrom: [oneBook getMeanPrice] 
                          asExternalValueUsing: minmax withShift: 0];
  // previousDayLiquidityQuantity
  // is the 6th input
  [dataVerificationMatrix R: 0 C: 5 setFrom:     
                          [self getLiquidityQuantity_target]   
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 5 setFrom: 
                          [self getLiquidityQuantity_target] 
                          asExternalValueUsing: minmax withShift: 0];
  // previousDayShareQuantity
  // is the 7th input
  [dataVerificationMatrix R: 0 C: 6 setFrom:     
                          [self getShareQuantity_target]   
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 6 setFrom: 
                          [self getShareQuantity_target] 
                          asExternalValueUsing: minmax withShift: 0];
// +++++++ end

  return self;
}

- setCT_Targets
{
  float corr, liquidityCorr, shareCorr,
  bPCTLiquidityQuantity_EO, bPCTShareQuantity_EO, bPCTMeanOperatingPrice;

  // NB. A general rule: IF WE NEED TO USE OTHER AGENTS OUTPUTS TO
  // GENERATE TARGETS WE MUST MANDATORYLY REFER TO OUTPUTS IN THE VERIFICATION
  // SET; we observe automatically this general rule if we build the getXXX
  // methods in interface objects as it is done in the introductory packages
  // bp-ct and ct-hayek, i.e. always getting inputs, outputs and targets
  // from the verification matrixes and - obviously - if we access values
  // only by means of those getXXX methods (addressed to other agents'
  // interfaces)


// +++++++ start

  // CT values (*)

  // effects

  // liquidityQuantity
  // is the target of the 1st effect (we don't need here to calculate
  // effects in the classic CT way, i.e. applying accounting rule to
  // action guesses, because the same calculations are made by the act2
  // step of the simulation framework
  bPCTLiquidityQuantity = [myAgentDataWarehouse getAgentLiquidityQuantity];  
  // learn to guess an effect close to the actual one

  // to deal here and below with liquidity, we need also the 
  // meanOperatingPrice value of the day
  bPCTMeanOperatingPrice= [myAgentDataWarehouse getAgentMeanOperatingPrice];

  // we have here an external objective (EO) as target that, in case, 
  // substitutes (see below) the original CT target
  bPCTLiquidityQuantity_EO=[self getPreviousDayLiquidityQuantity];
  bPCTLiquidityQuantity_EO+=eO_EPDelta*bPCTMeanOperatingPrice; 
                         // eO_EPDelta is expressed in unit of shares, so ...

  // checking minmax consistence
  if(bPCTLiquidityQuantity < [minmax R: 7 C: 0])
    bPCTLiquidityQuantity = [minmax R:7 C: 0];
  if(bPCTLiquidityQuantity > [minmax R: 7 C: 1])
    bPCTLiquidityQuantity = [minmax R:7 C: 1];

  if(bPCTLiquidityQuantity_EO < [minmax R: 7 C: 0])
    bPCTLiquidityQuantity_EO = [minmax R:7 C: 0];
  if(bPCTLiquidityQuantity_EO > [minmax R: 7 C: 1])
    bPCTLiquidityQuantity_EO = [minmax R:7 C: 1];


  // shareQuantity
  // is the target of the 2nd effect
  bPCTShareQuantity =  [myAgentDataWarehouse getAgentShareQuantity];          
  // learn to guess an effect close to the actual one

  // we have here an external objective (EO) as target that, in case, 
  // substitutes (see below) the original CT target
  bPCTShareQuantity_EO=[self getPreviousDayShareQuantity];
  bPCTShareQuantity_EO+=eO_EPDelta;

  // checking minmax consistence
  if(bPCTShareQuantity < [minmax R: 8 C: 0])
    bPCTShareQuantity = [minmax R:8 C: 0];
  if(bPCTShareQuantity > [minmax R: 8 C: 1])
    bPCTShareQuantity = [minmax R:8 C: 1];

  if(bPCTShareQuantity_EO < [minmax R: 8 C: 0])
    bPCTShareQuantity_EO = [minmax R:8 C: 0];
  if(bPCTShareQuantity_EO > [minmax R: 8 C: 1])
    bPCTShareQuantity_EO = [minmax R:8 C: 1];



  // buySell (>0 buy, <0 sell, =0 do nothing) 
  // is the target of the (1st) action


  // if liquidity guess (out or EO) > actual value, we have to decrease buy
  // guess or increase sell guess, accounting for the meanOperatingPrice
  if(useEO_EP==1 || useEO_EP==3)
  liquidityCorr = -(bPCTLiquidityQuantity_EO       -bPCTLiquidityQuantity)
                                                    / bPCTMeanOperatingPrice;
  else
  liquidityCorr = -([self getLiquidityQuantity_out]-bPCTLiquidityQuantity)
                                                    / bPCTMeanOperatingPrice;

  // if share guess (out or EO) > actual value, we have to increase buy
  // guess or decrease sell guess
  if(useEO_EP==2 || useEO_EP==3)
  shareCorr = bPCTShareQuantity_EO       -bPCTShareQuantity;
  else
  shareCorr = [self getShareQuantity_out]-bPCTShareQuantity;

  corr = liquidityCorr; if(fabs(corr)<fabs(shareCorr))corr=shareCorr;

  buySell = [self getBuySell_out] + corr;

  // checking minmax consistence
  if(buySell < [minmax R: 9 C: 0]) buySell = [minmax R:9 C: 0];
  if(buySell > [minmax R: 9 C: 1]) buySell = [minmax R:9 C: 1];



  // targets

  // liquidityQuantity (1st effect)
  [dataVerificationMatrix R: 0  C: 7    setFrom: bPCTLiquidityQuantity 
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 7 setFrom: bPCTLiquidityQuantity 
                          asExternalValueUsing: minmax withShift: 0];

  // shareQuantity (2nd effect)
  [dataVerificationMatrix R: 0  C: 8    setFrom: bPCTShareQuantity 
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 8 setFrom: bPCTShareQuantity 
                          asExternalValueUsing: minmax withShift: 0];

  // buySell (1st action)
  [dataVerificationMatrix R: 0  C: 9    setFrom: buySell 
                          asExternalValueUsing: minmax withShift: 0];
  [dataTrainingMatrix     R_lastAndC: 9 setFrom: buySell 
                          asExternalValueUsing: minmax withShift: 0];

// +++++++ end

  return self;
}

@end

// (*) first of all we define the CT targets; then, in a second step (after
// '// targets' comment), we report those values in the requested
// matrixes




