// **********************************
// from basic code to apps: keep this file unchanged
// (this warning comes from bp-ct-1.1, i.e the original application containing 
// this file)
// the file is modified here to use an ad hoc random distribution and the 
// 'used' var
// **********************************

// BPCTInterface.m                                        

#import "BPCTInterface.h"

// This is the basis of each agent interface; the adaptation to each speficic
// agent type are contained exclusively in specific interface objects,
// which inherit from this one (as an example, in the basic code distribution,
// we have SpecificInterface.h and .m)

@implementation BPCTInterface

// to find agent data (if the interface is never used, it returns 0)

- setUsed: (int) u
{
  if(used!=2)used=u; // if used has been set to 2, don't modify it
  return self;
}

  // the address of the book of the market
- setBook: b
{
  oneBook=b;
  return self;
}

- setDataWarehouse: dw
{
  myAgentDataWarehouse = dw;
  return self;
}

// in CT case, we can use External Objectives (EO) or External Porposal (EP)
// following a set of "if" instructions that use this variable in the
// specific interfaces

- setMyUniformIntRand: u
{
  myUniformIntRand = u;
  return self;
}

- setUseEO_EP: (int) eo
{
  useEO_EP=eo;
  return self;
}

- setEO_EPDelta: (float) eod
{
  eO_EPDelta=eod;
  return self;
}

- setAgentNumber: (int) n
{
  number=n;
  return self;
}

- createEnd
{
  [super createEnd];

  // automatic determination of parameters and matrix addresses via
  // myAgentDataWarehouse

  used=0; // the init value, before any step, influencing the results of the get
          //get methods addressed to inputs, outputs or target
  if([myAgentDataWarehouse getInitDone]!=0)used=2; // init values charged
                                     // nb both input and outputs-targets

 
  inputNodeNumber                = [myAgentDataWarehouse
                                    getInputNodeNumber];
  patternNumberInVerificationSet = [myAgentDataWarehouse
                                    getPatternNumberInVerificationSet];
  // array addresses from datawarehouse
  outputVerificationMatrix = [myAgentDataWarehouse
                                              getOutputVerificationMatrix];
  targetVerificationMatrix = [myAgentDataWarehouse
                                              getTargetVerificationMatrix];
  dataVerificationMatrix   = [myAgentDataWarehouse
                                              getDataVerificationMatrix];
  dataTrainingMatrix       = [myAgentDataWarehouse
                                              getDataTrainingMatrix];
  minmax                   = [myAgentDataWarehouse
                                              getMinmax];

  return self;
}

- (int) getAgentNumber
{
  return number;
}


  // the following methods are left intentionally void; they will be
  // overridden in the other XXXInterface objects that we will generate
  // when using the CT scheme or simply an internal automatic input
  // and target generation process as in SpecificInterface.h
  // and SpecificInterface.m in the basic code distribution

  // they are needed here because, to keep Agent.h and Agent.m unchanged,
  // we import, in Agent.h, Interface.h as a type for myInterface, while the
  // agent instance will receive the address of its proper specific
  // interface:
  // if the type Inteface would not respond to the method reported below,
  // we should receive a few warning messages during the compilation


- initialize 
{
  int i;
  int outputNodeNumber;
  // inputNodeNumberLocal is initialized in createEnd
  outputNodeNumber=[myAgentDataWarehouse getOutputNodeNumber];

  // the following lines are useful only in CT scheme if we look (via get...)
  // for initial values in dataVerificationMatrix, targetVerificationMatrix,
  // outputVerificationMatrix (initial values are
  // stored in dataTrainingMatrix via DataWarehouse createEnd method)

  for (i=0;i<inputNodeNumber+outputNodeNumber;i++)
  [dataVerificationMatrix R:0 C: i setFrom:
                            [dataTrainingMatrix R:0 C: i]];
  for (i=0;i<outputNodeNumber;i++)
  [outputVerificationMatrix R:0 C: i setFrom:
                            [dataTrainingMatrix R:0 C: i+inputNodeNumber]];
  for (i=0;i<outputNodeNumber;i++)
  [targetVerificationMatrix R:0 C: i setFrom:
                            [dataTrainingMatrix R:0 C: i+inputNodeNumber]];

  return self;
}


- setCT_Inputs
{
  return self;
}

- setCT_Targets
{
  return self;
}

@end



