// ForecastingAgent.h

#import <objectbase/SwarmObject.h>
#import <random.h>
#import "Book.h"
#import "Matrix2.h"
#import <simtools.h>      // necessary to invoke ObjectLoader
#import <simtoolsgui/GUISwarm.h> // to use CREATE_ARCHIVED_PROBE_DISPLAY
#import <simtoolsgui.h>          // ""

@interface ForecastingAgent: SwarmObject
{

  int dataWindowLength, nAheadForecasting, forecastingTrainingSetLength,
      forecastHistoryLength,
      currentDay, printing,
      epochNumberInEachForecastingTrainingCycle,
      learningProcessEveryNDays, cleanForecastingANNEveryMgtemNDays,
      inputNodeNumber1,
      hiddenNodeNumber, hiddenNodeNumber1, outputNodeNumber,
      learningProcessTrue, number;

  Matrix2 * forecastingTrainingSet, * forecastingSet,
          * forecastHistory,
          * minmax,  * outputTrainingMatrix,
          * targetTrainingMatrix,
          * who, * dwho, * wih, * dwih,
          * outputVerificationMatrix, * targetVerificationMatrix,
          * inputLayer, * hiddenLayer, * outputLayer, * deltaOut,
          * hiddenLayerTransFuncDerivatives, * outputLayerTransFuncDerivatives;

  float weightRange, eps, alpha, index;
  int usingRandomOrderInLearning;

  Book * oneBook;
  id modelSwarmAddress;
  id simpleANNRuleMasterAddress;
  id <UniformDoubleDist> myUniformDblRand;
}

- createEnd;

- setNumber: (int) n;
- setModelSwarmAddress: m;
- setSimpleANNRuleMasterAddress: r;
- setBook: b;
- setMyUniformDblRand: u; // an ad hoc distribution, see comment in
                          // modelSwarm.h and .m

// set the length of the window containing the data of the past for each
// training pattern; the position n of the day ahead that we are forecasting;
// the length of the training set; the epoch number in learning process;
// the repetition of learning process and repeated cleaning of the ANN
- setDataWindowLength: (int) dwl andNAheadForecasting: (int) n
                                 andForecastingTrainingSetLength: (int) tsl
                    andEpochNumberInEachForecastingTrainingCycle: (int) en
                    andLearningProcessEveryNDays: (int) nd
                    andCleanForecastingANNEveryMgtemNDays: (int) md
                    andForecastHistoryLength: (int) hl;

- setPrinting: (int) p;

- getForecastingSet;
- getForecastingTrainingSet;
- getMinmax;
- (int) getDataWindowLength;
- (int) getHiddenNodeNumber;
- (int) getForecastingTrainingSetLength;

-(float) getEps;
-(float) getAlpha;
-(int)   getRandomOrderInLearning;

- (int) getEpochNumberInEachForecastingTrainingCycle;

- getOutputTrainingMatrix;
- getTargetTrainingMatrix;

- getInputLayer;
- getHiddenLayer;
- getHiddenLayerTransFuncDerivatives;
- getOutputLayer;
- getOutputLayerTransFuncDerivatives;
- getDeltaOut;

- getWih;
- getDwih;
- getWho;
- getDwho;

- step;

- (float) getPastEstimateIndex;
- (float) getCurrentEstimateIndex;
- (float) getBackPropagationErrorInTrainingSet;
- (float) getProportionalErrorInTrainingSet;

@end




