// ModelSwarm.h
#import <objectbase/Swarm.h>
#import <simtools.h>      // necessary to invoke ObjectLoader
#import <objectbase.h>    // needed by <ProbeMap> in ModelSwarm.m
#import <activity.h>
#import <collections.h>

  // agentForge step 2

#import "AvatarAgent.h"
#import "BasicSumAgent.h"
#import "RandomAgent.h"
#import "CurrentAgent.h"
#import "EventAgent.h"
#import "WasteTimeAgent.h"
#import "MarketImitatingAgent.h"
#import "LocallyImitatingAgent.h"
#import "StopLossAgent.h"
#import "ANNForecastAppAgent.h"
#import "ForecastingAgent.h"
#import "BPCTAgentA.h"
#import "BPCTAgentAInterface.h" // related to the agents used in our simulation
#import "BPCTAgentB.h"
#import "BPCTAgentBInterface.h" // related to the agents used in our simulation
#import "BPCTPriceRuleMaster.h"

#import "ArbitrageurAgent.h"
#import "IndexCalculator.h"

#import "Book.h"
#import "EventGenerator.h"
#import "EventRuleMaster.h"
#import "RandomRuleMaster.h"
#import "StopLossRuleMaster.h"
#import "SimpleANNRuleMaster.h"
#import "SimpleANNRuleMaker.h"
#import "MatrixMult.h"
#import "VectorTransFunc.h"
#import "TransFunc.h"
#import "Quota.h"
#import <random.h> // to generate ad hoc private distributions to be used
                   // in SimpleANN and BPCT, to avoid interferences with random
                   // sequences used in determining agent behavior
#import "BPCTRuleMaster.h"
#import "BPCTRuleMaker.h"
#import "BPCTDataWarehouse.h"

@interface ModelSwarm: Swarm
{
  int bookNumber,futureBook, dayNumber, maxOrderQuantity,maxOrderQuantity2, priceVolumesHistoryLength,
     quantityVolumesHistoryLength, meanPriceHistoryLength,humanPenalty,
      localHistoryLength, stopLossInterval, checkingIfShortOrLong, printing, delay,
      dataWindowLength, nAheadForecasting, forecastingTrainingSetLength,
      epochNumberInEachForecastingTrainingCycle, learningProcessEveryNDays,delayInAuction,
      cleanForecastingANNEveryMgtemNDays, wasteTimeAgentNumber, arbitrageur, status, arbitrageurAgentNumber, arbitrageurGain,
      arbitrageurOperatingIntervalFixed,continuosMarket, eventType, eventAgentChoiceToActBeforeOpening;
  float asymmetricBuySellProb, agentProbToActBeforeOpening,
        minCorrectingCoeff, maxCorrectingCoeff,
        asymmetricRange, floorP, agentProbToActBelowFloorP,
        maxLossRate, arbitrageurOperatingInterval ,
        eventChangeProbability, eventSensibility,
        aNNInactivityRange, aNNForecastAppAgentActDailyProb;

  // agentForge step 3
  int   agentNumber, randomAgentNumber, marketImitatingAgentNumber, avatarAgentNumber,
    locallyImitatingAgentNumber, stopLossAgentNumber, eventAgentNumber	,
    aNNForecastAppAgentNumber, bPCTAgentANumber,
    bPCTAgentAEO_EP_0_Number, bPCTAgentAEO_EP_1_Number,
    bPCTAgentAEO_EP_2_Number, bPCTAgentAEO_EP_3_Number,
                               bPCTAgentBNumber,
    bPCTAgentBEO_EP_0_Number, bPCTAgentBEO_EP_1_Number,
    bPCTAgentBEO_EP_2_Number, bPCTAgentBEO_EP_3_Number;
  id <List> bookList, agentList, randomAgentList, marketImitatingAgentList,
            locallyImitatingAgentList, stopLossAgentList, aNNForecastAppAgentList,
            bPCTAgentAList, bPCTAgentBList, avatarAgentList, eventAgentList,
            forecastingAgentList,arbitrageurAgentList,wasteTimeAgentList;

  id <Array> bPCTAgentAArray;
  id <Index> bPCTAgentAArrayIndex;

  id <Array> bPCTAgentBArray;
  id <Index> bPCTAgentBArrayIndex;

  id <Array> agentArray;
  id <Index> agentArrayIndex;
  id <Array> forecastingAgentArray;
  id <Index> forecastingAgentArrayIndex;
  id <Array> bookArray;
  id <Index> bookArrayIndex;
  id <ListShuffler> listShuffler;

  id <ActionGroup> modelActions1, modelActions2, modelActions3, modelActions4;
  id <Schedule> modelSchedule;

  // agentForge step 3r

  // see above about private distributions
  id <SimpleRandomGenerator> myGenerator;
  id <UniformDoubleDist> myUniformDblRand;
  id <UniformIntegerDist> myUniformIntRand;
  id <NormalDist> myNormalDistRand;
  id <SimpleRandomGenerator> myGenerator2;
  id <UniformDoubleDist> myUniformDblRand2;
  id <UniformIntegerDist> myUniformIntRand2;
  id <SimpleRandomGenerator> myGenerator3;
  id <UniformDoubleDist> myUniformDblRand3;
  id <UniformIntegerDist> myUniformIntRand3;

  Book * aBook;

  EventRuleMaster * eventRuleMaster;
  RandomRuleMaster * randomRuleMaster;
  IndexCalculator * indexCalculator;

  CurrentAgent * theCurrentAgent;
  ForecastingAgent * aForecastingAgent;
  StopLossRuleMaster * stopLossRuleMaster;
  SimpleANNRuleMaster * simpleANNRuleMaster;
  SimpleANNRuleMaker * simpleANNRuleMaker;
  MatrixMult * matrixMult;
  VectorTransFunc * vectorTransFunc;
  TransFunc * transFunc;
  EventGenerator * eventGenerator;
  Quota * quota;


  // BPCT

  // agentForge step 3b
  int epochNumberInEachBPCTTrainingCycle;
  float agentAEO_EPDelta, agentBEO_EPDelta;

  // we are creating independent RuleMaster/Maker for each type of
  // BPCT agent, to have separated random distributions also in weight
  // generation
  BPCTRuleMaster * bPCTRuleMasterA, * bPCTRuleMasterB;
  BPCTRuleMaker * bPCTRuleMakerA, * bPCTRuleMakerB;
  BPCTPriceRuleMaster * bPCTPriceRuleMasterA, * bPCTPriceRuleMasterB;

  // agentForge step 3c
  int bPCTAgentAInputNodeNumber, bPCTAgentAHiddenNodeNumber,
      bPCTAgentAOutputNodeNumber,
      bPCTAgentBInputNodeNumber, bPCTAgentBHiddenNodeNumber,
      bPCTAgentBOutputNodeNumber,

      bPCTPatternNumberInVerificationSet,
      bPCTPatternNumberInTrainingSet, bPCTAgentsAreDisplayingData,
      usingRandomOrderInBPCTLearning,
      longTermLearningInBPCT_OnlyWithCompleteTrainingSet,
      useOutputsAsTargetsInBPCT_RelearningScheme;
  float bPCTWeightRange, bPCTEps, bPCTAlpha;

}


+ createBegin: aZone;
- createEnd;
- buildObjects;
- buildActions;
- activateIn: swarmContext;

- increaseCurrentDayNumber;
- (int) getCurrentDay;
- check24hSpent;
- getAgentArrayIndex;
// - getForecastAgentArrayIndex;
- getBookArrayIndex;
- getBPCTAgentAArrayIndex;
- getBPCTAgentBArrayIndex;
- getBook: (int) n;
- (int) getBookNumber;
- getEventGenerator;

  // agentForge step 4
- (int) getBPCTAgentANumber;
- getBPCTAgentAArrayIndex;
- (int) getBPCTAgentBNumber;
- getBPCTAgentBArrayIndex;
- getAgentList;

- getRandomAgentList;
- getWasteTimeAgentList;
- getArbitrageurAgentList;
- getMarketImitatingAgentList;
- getLocallyImitatingAgentList;
- getStopLossAgentList;
- getANNForecastAppAgentList;
- getBPCTAgentAList;
- getBPCTAgentBList;
- getAvatarAgentList;
- getEventAgentList;
- getForecastingAgent: (int) n;
- getIndexCalculator;
- openProbeTo: (int) ag;
- setDayStateOpeningAuction;
- setDayStateContinuousTrading;
- setDayStateClosingAuction;
- waitForOrders;

@end


