// ModelSwarm.m

#import "ModelSwarm.h"
#include <time.h>
//#include <unistd.h>

@implementation ModelSwarm

+ createBegin: aZone
{
  ModelSwarm *obj;
  id <ProbeMap> probeMap;
  time_t t;
  int timestamp;
  timestamp=time (&t);


  // in createBegin, we set up the simulation parameters

  // first, call our superclass createBegin - the return value is the
  // allocated Swarm object.

  obj = [super createBegin: aZone];

  // now fill in simulation parameters with default values.

  // agentForge step 5
  obj-> bookNumber			   =  3;
  obj-> continuosMarket			   =  1; // the model uses last prices and wealth as starting point for the new run
  obj-> humanPenalty			   =  2;
  obj-> arbitrageurAgentNumber		   =  1;
  obj-> arbitrageurOperatingInterval       =  0.1;
  obj-> arbitrageurOperatingIntervalFixed  =  1;
  obj-> arbitrageurGain                    =  1;
  obj-> eventAgentNumber                   =  15;
  obj-> randomAgentNumber                  =  200;
  obj-> marketImitatingAgentNumber         =  0;
  obj-> locallyImitatingAgentNumber        =  0;
  obj-> stopLossAgentNumber                =  0;
  obj-> avatarAgentNumber                  =  170;
  obj-> wasteTimeAgentNumber               =  100;
  obj-> aNNForecastAppAgentNumber          =  0;
  obj-> bPCTAgentAEO_EP_0_Number           =  0;
  obj-> bPCTAgentAEO_EP_1_Number           =  0;
  obj-> bPCTAgentAEO_EP_2_Number           =  0;
  obj-> bPCTAgentAEO_EP_3_Number           =  0;
  obj-> bPCTAgentBEO_EP_0_Number           =  0;
  obj-> bPCTAgentBEO_EP_1_Number           =  0;
  obj-> bPCTAgentBEO_EP_2_Number           =  0;
  obj-> bPCTAgentBEO_EP_3_Number           =  0;

  obj->bPCTAgentANumber = obj->bPCTAgentAEO_EP_0_Number + obj->bPCTAgentAEO_EP_1_Number
                        + obj->bPCTAgentAEO_EP_2_Number + obj->bPCTAgentAEO_EP_3_Number;
  obj->bPCTAgentBNumber = obj->bPCTAgentBEO_EP_0_Number + obj->bPCTAgentBEO_EP_1_Number
                        + obj->bPCTAgentBEO_EP_2_Number + obj->bPCTAgentBEO_EP_3_Number;

  obj-> agentNumber = obj-> randomAgentNumber + obj-> marketImitatingAgentNumber +
                      obj-> locallyImitatingAgentNumber + obj->stopLossAgentNumber +
                      obj-> aNNForecastAppAgentNumber + obj->bPCTAgentANumber +
                      obj -> bPCTAgentBNumber + obj -> avatarAgentNumber +
                      obj-> eventAgentNumber +   obj-> wasteTimeAgentNumber +
                      obj-> arbitrageurAgentNumber ;
  // repeat this sum below, in BuildObjects method

  obj-> dayNumber                          =  0;
  obj-> asymmetricBuySellProb              =  0.9;
  obj-> minCorrectingCoeff                 =  0.9;
  obj-> maxCorrectingCoeff                 =  1.1;
  obj-> asymmetricRange                    =  0.0;
  obj-> agentProbToActBeforeOpening        =  0.25;
  obj-> floorP                             =  0.3;
  obj-> agentProbToActBelowFloorP          =  0.5;
  obj-> eventType                          =  1;
  obj-> eventChangeProbability             =  0.003;
  obj-> eventSensibility                   =  0.20;
  obj-> eventAgentChoiceToActBeforeOpening =  1;
  obj-> maxOrderQuantity                   =  1; // max order number per agent

  // agentForge step 6
  // used by imitating agents
  obj-> meanPriceHistoryLength             =  200;
  obj-> priceVolumesHistoryLength          =  200;
  obj-> quantityVolumesHistoryLength       =  200;
  obj-> localHistoryLength                 =  20;
  // used by stop loss agent
  obj-> maxLossRate                        =  0.10;
  obj-> stopLossInterval                   =  2;
  obj-> checkingIfShortOrLong              =  1;
  // used by forecasting agent
  obj-> dataWindowLength                   =  30;
  obj-> nAheadForecasting                  =  10;
  // used by forecasting agent
  obj-> forecastingTrainingSetLength       =  100;
  obj->
 epochNumberInEachForecastingTrainingCycle =  100;
  obj-> learningProcessEveryNDays          =  10;
  obj-> cleanForecastingANNEveryMgtemNDays =  50;
  // used by agents applying ANN forecast
  obj-> aNNInactivityRange                 =  0.02;
  obj-> aNNForecastAppAgentActDailyProb    =  0.1;
  // used by agent of BPCT type
  obj->epochNumberInEachBPCTTrainingCycle  =  100;
  obj->agentAEO_EPDelta                    =  0.1;
  obj->agentBEO_EPDelta                    =  10;

  obj-> printing                           =  0; // if 1 many objects print
                                                 // data on the terminal
                                                 // window; if 2 only
                                                 // forecastingAgent prints;
                                                 // forecastinAgent uses also 3;
                                                 // 4 is used in BasicSumAgent;
                                                 // 5 in ANNForecastAppAgent;

  obj-> delay                              =  5; // if delay>0 the random agent execute a for cicle to delay the execution of the simualtion for n seconds, where n is the number inticated
  obj-> delayInAuction			   =  200; // if delayInAuction>0 the system wait n second during the auctions
  // build a customized probe map. Without a probe map, the default
  // is to show all variables and messages. Here we choose to
  // customize the format of the probe to give a nicer interface.

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bookNumber"                  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "continuosMarket"             inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "humanPenalty"             inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "arbitrageurAgentNumber"      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "arbitrageurOperatingInterval" inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "arbitrageurOperatingIntervalFixed" inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "arbitrageurGain" inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "randomAgentNumber"           inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "avatarAgentNumber"           inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "marketImitatingAgentNumber"  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "locallyImitatingAgentNumber" inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "stopLossAgentNumber"         inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "eventAgentNumber"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "wasteTimeAgentNumber"        inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "aNNForecastAppAgentNumber"   inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentAEO_EP_0_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentAEO_EP_1_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentAEO_EP_2_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentAEO_EP_3_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentBEO_EP_0_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentBEO_EP_1_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentBEO_EP_2_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "bPCTAgentBEO_EP_3_Number"    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "agentNumber"                 inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "asymmetricBuySellProb"       inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "minCorrectingCoeff"          inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "maxCorrectingCoeff"          inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "asymmetricRange"             inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "agentProbToActBeforeOpening" inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "floorP"                      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "agentProbToActBelowFloorP"   inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "eventType"                   inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "eventChangeProbability"      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "eventSensibility"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "eventAgentChoiceToActBeforeOpening"  inClass: [self class]]];

  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "maxOrderQuantity"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "meanPriceHistoryLength"      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "priceVolumesHistoryLength"   inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "quantityVolumesHistoryLength"
   													  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "localHistoryLength"          inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "maxLossRate"                 inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "stopLossInterval"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "checkingIfShortOrLong"       inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "dataWindowLength"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "nAheadForecasting"           inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable:"forecastingTrainingSetLength" inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable:"epochNumberInEachForecastingTrainingCycle"
                                                      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "learningProcessEveryNDays"   inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "cleanForecastingANNEveryMgtemNDays"
                                                      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "aNNInactivityRange"          inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "aNNForecastAppAgentActDailyProb"
                                                      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "epochNumberInEachBPCTTrainingCycle"
                                                      inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "agentAEO_EPDelta"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "agentBEO_EPDelta"            inClass: [self class]]];

  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "printing"                    inClass: [self class]]];

  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "delay"                       inClass: [self class]]];

  [probeMap addProbe: [probeLibrary
   getProbeForVariable: "delayInAuction"              inClass: [self class]]];

  [probeMap addProbe: [probeLibrary
   getProbeForMessage:  "openProbeTo:"                inClass: [self class]]];


  // Now install our custom probeMap into the probeLibrary.

  [probeLibrary setProbeMap: probeMap For: [self class]];

  // for generating run with different seed decomment the line below
  [randomGenerator setStateFromSeed:  timestamp];
  //  for more information on random numbers generation see tha appendix in Swarm Reference Book

  return obj;
}

- createEnd
{
  return [super createEnd];
}


- buildObjects
{
  int i;
  float otherAgents;

  // agentForge step 7
  RandomAgent * anAgent1;
  MarketImitatingAgent * anAgent2;
  LocallyImitatingAgent * anAgent3;
  StopLossAgent * anAgent4;
 ANNForecastAppAgent * anAgent5;
 BPCTAgentA * anAgent6;
 BPCTAgentB * anAgent7;
  AvatarAgent * anAgent8;
  ArbitrageurAgent * anAgent9;
  EventAgent * anAgent10;
  WasteTimeAgent * anAgent11;

BPCTAgentAInterface * anInterface6;
 BPCTAgentBInterface * anInterface7;

BPCTDataWarehouse * aDataWarehouse;

  // BPCT specific
  char * bPCTMinmaxFileNameA      ="minmaxA.data",  // mandatory, one for each
                                                    // BPCT agent type
       * bPCTMinmaxFileNameB      ="minmaxB.data",

       * bPCTInitValuesFileNameA  ="initA.val",     // this file is used only
                                                    // with internal data
                                                    // generation (CT scheme)
                                                    // but anyway the existence
                                                    // of the file is not
                                                    // mandatory
       * bPCTInitValuesFileNameB  ="initB.val",
       * unusedFile              ="";
  // BPCT end

  // creating tools to generate ad hoc distributions to be used
  // in SimpleANN and BPCT, to avoid interferences with random
  // sequences used in determining agent behavior

  // agentForge step 7r

  unsigned int mySeed, mySeed2, mySeed3;
  // used by forecastingAgent
  mySeed = 223776;
  myGenerator = [MT19937gen create: self setStateFromSeed: mySeed];
              // MT19937gen is the generator internally used for default call
              // to Dbl and Int uniform distributions
  myUniformDblRand = [UniformDoubleDist create: self
		                                setGenerator: myGenerator];
  myUniformIntRand = [UniformIntegerDist create: self
		                                setGenerator: myGenerator];
  myNormalDistRand = [NormalDist create: self   setGenerator: myGenerator];
  // used by BPCTAgentA
  mySeed2 = 112233;
  myGenerator2 = [MT19937gen create: self setStateFromSeed: mySeed2];
              // MT19937gen is the generator internally used for default call
              // to Dbl and Int uniform distributions
  myUniformDblRand2 = [UniformDoubleDist create: self
		                                setGenerator: myGenerator2];
  myUniformIntRand2 = [UniformIntegerDist create: self
		                                setGenerator: myGenerator2];

  // used by BPCTAgentB
  mySeed3 = 333112;
  myGenerator3 = [MT19937gen create: self setStateFromSeed: mySeed3];
              // MT19937gen is the generator internally used for default call
              // to Dbl and Int uniform distributions
  myUniformDblRand3 = [UniformDoubleDist create: self
		                                setGenerator: myGenerator3];
  myUniformIntRand3 = [UniformIntegerDist create: self
		                                setGenerator: myGenerator3];

  // agentForge step 5b
  // this is an operating second definition of the followin sums (see above in
  // obj-> agentNumber assignement

  bPCTAgentANumber = bPCTAgentAEO_EP_0_Number + bPCTAgentAEO_EP_1_Number
                   + bPCTAgentAEO_EP_2_Number + bPCTAgentAEO_EP_3_Number;

  bPCTAgentBNumber = bPCTAgentBEO_EP_0_Number + bPCTAgentBEO_EP_1_Number
                   + bPCTAgentBEO_EP_2_Number + bPCTAgentBEO_EP_3_Number;

  agentNumber = randomAgentNumber + marketImitatingAgentNumber +
                locallyImitatingAgentNumber + stopLossAgentNumber +
                aNNForecastAppAgentNumber+bPCTAgentANumber+bPCTAgentBNumber+
                avatarAgentNumber+arbitrageurAgentNumber+eventAgentNumber+wasteTimeAgentNumber;

  // if dataWindowLength+nAheadForecasting > meanPriceHistoryLength
  // a matrix error arises
  if(dataWindowLength+2*nAheadForecasting>meanPriceHistoryLength)
  {
    printf("The sum of dataWindowLength plus 2*nAheadForecasting is\n"
           "greater than meanPriceHistoryLength; this is a nonsense.\n"
           "See the comment 'NB about lagged values and return indexes' in\n"
           "ForecastingAgent.m\n");
    exit(0);
  }

  // randomRuleMaster
  randomRuleMaster=[RandomRuleMaster createBegin: self];
  [randomRuleMaster setAgentProbToActBeforeOpening:
                                  agentProbToActBeforeOpening];
  [randomRuleMaster setMinCorrectingCoeff: minCorrectingCoeff];
  [randomRuleMaster setMaxCorrectingCoeff: maxCorrectingCoeff];
  [randomRuleMaster setAsymmetricRange: asymmetricRange];
  [randomRuleMaster setFloorP: floorP
              andAgentProbToActBelowFloorP: agentProbToActBelowFloorP];
  randomRuleMaster=[randomRuleMaster createEnd];

  // eventRuleMaster
  eventRuleMaster=[EventRuleMaster createBegin: self];
  [eventRuleMaster setAgentProbToActBeforeOpening:
                                  agentProbToActBeforeOpening];
  if (eventAgentChoiceToActBeforeOpening!=0 && eventAgentChoiceToActBeforeOpening!=1)
					    {printf ("The eventAgentChoiceToActBeforeOpening must be 1 or 0. \n");
                                  eventAgentChoiceToActBeforeOpening=1;
                                  }
  [eventRuleMaster setEventAgentChoiceToActBeforeOpening:
					    eventAgentChoiceToActBeforeOpening];
  [eventRuleMaster setMinCorrectingCoeff: minCorrectingCoeff];
  [eventRuleMaster setMaxCorrectingCoeff: maxCorrectingCoeff];
  [eventRuleMaster setAsymmetricRange: asymmetricRange];
  if (eventSensibility<0){printf ("The eventSensibility must be greater than zero. \n");
                                  eventSensibility=0.20;
                                  }
  [eventRuleMaster setEventSensibility: eventSensibility];
  eventRuleMaster=[eventRuleMaster createEnd];


  // stopLossRuleMaster
  stopLossRuleMaster=[StopLossRuleMaster createBegin: self];
  [stopLossRuleMaster setAgentProbToActBeforeOpening:
                                  agentProbToActBeforeOpening];
  [stopLossRuleMaster setMinCorrectingCoeff: minCorrectingCoeff];
  [stopLossRuleMaster setMaxCorrectingCoeff: maxCorrectingCoeff];
  [stopLossRuleMaster setAsymmetricRange: asymmetricRange];
  [stopLossRuleMaster setFloorP: floorP
              andAgentProbToActBelowFloorP: agentProbToActBelowFloorP];
  stopLossRuleMaster=[stopLossRuleMaster createEnd];

  listShuffler=[ListShuffler createBegin: self];
  listShuffler=[listShuffler createEnd];

  bookList=[List create: self];
  arbitrageurAgentList=[List create: self];
  // agentForge step 5bb
  agentList=[List create: self];
  randomAgentList=[List create: self];
  avatarAgentList=[List create: self];
  marketImitatingAgentList=[List create: self];
  locallyImitatingAgentList=[List create: self];
  eventAgentList=[List create: self];
  stopLossAgentList=[List create: self];
  eventAgentList=[List create: self];
  aNNForecastAppAgentList=[List create: self];
  bPCTAgentAList=[List create: self];
  bPCTAgentBList=[List create: self];
  forecastingAgentList=[List create: self];
  wasteTimeAgentList=[List create: self];

  agentArray = [Array create: self];
  if (agentNumber==0) {printf("Nonsense: agentNumber cannot be 0");exit(0);}
  [agentArray setCount: agentNumber];
  agentArrayIndex = [agentArray begin: self];

  // agentForge step 5bbb

  if (bPCTAgentANumber!=0)
  {
   bPCTAgentAArray = [Array create: self];
   [bPCTAgentAArray setCount: bPCTAgentANumber];
   bPCTAgentAArrayIndex = [bPCTAgentAArray begin: self];
  }

  if (bPCTAgentBNumber!=0)
  {
   bPCTAgentBArray = [Array create: self];
   [bPCTAgentBArray setCount: bPCTAgentBNumber];
   bPCTAgentBArrayIndex = [bPCTAgentBArray begin: self];
  }


  if (bookNumber<1){printf("Nonsense: You need almost ONE book (internally set to 1)\n");
                                  bookNumber=1;
                                  }

  bookNumber++; // add one book for future

  if (arbitrageurAgentNumber>=1){
                   futureBook=bookNumber; // the futureBook is the last one
                    }else{futureBook=0;}

  bookArray = [Array create: self];
  [bookArray setCount: bookNumber];
  bookArrayIndex = [bookArray begin: self];

  forecastingAgentArray = [Array create: self];
  [forecastingAgentArray setCount: bookNumber];
  forecastingAgentArrayIndex = [forecastingAgentArray begin: self];

  maxOrderQuantity2=2*agentNumber*bookNumber;

   // IndexCalculator

  indexCalculator=[IndexCalculator createBegin: self];
  [indexCalculator setBookArrayIndex: bookArrayIndex];
  [indexCalculator setBookNumber: bookNumber];
  [indexCalculator setPrinting: printing];
  indexCalculator=[indexCalculator createEnd];



  for (i=1;i<=bookNumber;i++)
  {
   aBook = [Book createBegin: self];
   [aBook setNumber: i];
   [aBook setContinuosMarket: continuosMarket];
   [aBook setFutureBook: futureBook];
   [aBook setIndexCalculator: indexCalculator];
   [aBook setAgentArrayIndex: agentArrayIndex];
   [aBook setNAheadForecasting: nAheadForecasting];
   [aBook setAgentNumber: agentNumber];            // to build the matrixes
   [aBook setMaxOrderQuantity: maxOrderQuantity2];  // "   "
    if (meanPriceHistoryLength<2){printf("The length of the history of mean "
                                         "prices cannot be <2 (internally "
                                         "set to 2).\n");
                                  meanPriceHistoryLength=2;
                                  }
   [aBook setMeanPriceHistoryLength: meanPriceHistoryLength];
    if (priceVolumesHistoryLength<2){printf("The length of the history of Volumes "
                                         "cannot be <2 (internally "
                                         "set to 2).\n");
                                  priceVolumesHistoryLength=2;
                                  }
   [aBook setPriceVolumesHistoryLength: priceVolumesHistoryLength];
    if (quantityVolumesHistoryLength<2){printf("The length of the history of Volumes "
                                         "cannot be <2 (internally "
                                         "set to 2).\n");
                                  quantityVolumesHistoryLength=2;
                                  }
   [aBook setQuantityVolumesHistoryLength: quantityVolumesHistoryLength];
    if (localHistoryLength<1){printf    ("The length of the local history"
                                         "cannot be <1       (internally "
                                         "set to 1).\n");
                                  localHistoryLength=1;
                                  }
   [aBook setLocalHistoryLength: localHistoryLength];
   [aBook setPrinting: printing];
   aBook = [aBook createEnd];

   [bookList addLast: aBook];
   [bookArrayIndex next];
   [bookArrayIndex put: aBook];

  }





  //eventGenerator
  eventGenerator = [EventGenerator createBegin: self];
    if (eventType!=0 && eventType!=1){printf ("The eventType must value 0 or 1. \n");
                                  eventType=0;
                                  }
  [eventGenerator setEventType: eventType];
  [eventGenerator setStart: 0];
  [eventGenerator setMyNormalDistRand: myNormalDistRand];
    if (eventChangeProbability>1 || eventChangeProbability<0){printf ("The eventChangeProbability must be"
												"set between 0.00 and 1.00. \n");
                                  eventChangeProbability=0.15;
                                  }
  [eventGenerator setEventChangeProbability: eventChangeProbability];
  [eventGenerator setAvatarAgentList: avatarAgentList];
  eventGenerator = [eventGenerator createEnd];




  // a few checks
    if (asymmetricBuySellProb<0.5){printf("The asymmetricBuySellProb "
                                         "cannot be < 0.5 (internally "
                                         "set to 0.5).\n");
                                  asymmetricBuySellProb=0.5;
                                  }

    if (stopLossInterval>meanPriceHistoryLength)
                                  {printf("stopLossInterval "
                                         "cannot be > meanPriceHistoryLength"
                                         "\n(internally "
                                         "set to meanPriceHistoryLength).\n");
                                  stopLossInterval=meanPriceHistoryLength;
                                  }







  // randomAgent
  for (i=1;i<=randomAgentNumber;i++)
  {
  anAgent1=[RandomAgent createBegin: self];
  [anAgent1 setNumber: i];
  [anAgent1 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent1 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent1 setBookArrayIndex: bookArrayIndex];
  [anAgent1 setBookNumber: bookNumber];
  [anAgent1 setRuleMaster: randomRuleMaster];
  [anAgent1 setPrinting: printing];
  [anAgent1 setDelay: delay];
  anAgent1=[anAgent1 createEnd];

  [agentList addLast: anAgent1];
  [randomAgentList addLast: anAgent1];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent1];
  }

  // marketImitatingAgent
  for (i=randomAgentNumber+1;i<=randomAgentNumber + marketImitatingAgentNumber;i++)
  {
  anAgent2=[MarketImitatingAgent createBegin: self];
  [anAgent2 setNumber: i];
  [anAgent2 setAsymmetricBuySellProb: asymmetricBuySellProb];
  [anAgent2 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent2 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent2 setBookArrayIndex: bookArrayIndex];
  [anAgent2 setBookNumber: bookNumber];
  [anAgent2 setRuleMaster: randomRuleMaster];
  [anAgent2 setPrinting: printing];
  anAgent2=[anAgent2 createEnd];

  [agentList addLast: anAgent2];
  [marketImitatingAgentList addLast: anAgent2];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent2];
  }

  // locallyImitatingAgent
  for (i=randomAgentNumber+marketImitatingAgentNumber+1;
       i<=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber;i++)
  {
  anAgent3=[LocallyImitatingAgent createBegin: self];
  [anAgent3 setNumber: i];
  [anAgent3 setAsymmetricBuySellProb: asymmetricBuySellProb];
  [anAgent3 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent3 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent3 setBookArrayIndex: bookArrayIndex];
  [anAgent3 setBookNumber: bookNumber];
  [anAgent3 setRuleMaster: randomRuleMaster];
  [anAgent3 setPrinting: printing];
  anAgent3=[anAgent3 createEnd];

  [agentList addLast: anAgent3];
  [locallyImitatingAgentList addLast: anAgent3];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent3];
  }

  // stopLossAgent
  for (i=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber+1;
       i<=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber+
          stopLossAgentNumber;i++)
  {
  anAgent4=[StopLossAgent createBegin: self];
  [anAgent4 setNumber: i];
  [anAgent4 setAsymmetricBuySellProb: asymmetricBuySellProb];
  [anAgent4 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent4 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent4 setStopLossInterval: stopLossInterval];
  [anAgent4 setMaxLossRate: maxLossRate
            andCheckingIfShortOrLong: checkingIfShortOrLong];

  int randomBookNumber;
  randomBookNumber=[uniformIntRand getIntegerWithMin: 1 withMax: bookNumber];
  Book * oneBook;
  [bookArrayIndex setOffset: randomBookNumber-1];
  oneBook=[bookArrayIndex get];
  [anAgent4 setBook: oneBook];
  [anAgent4 setRuleMaster: stopLossRuleMaster];
  [anAgent4 setPrinting: printing];
  anAgent4=[anAgent4 createEnd];

  [agentList addLast: anAgent4];
  [stopLossAgentList addLast: anAgent4];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent4];
  }




  // we must create here a lot of objects before anAgent5, because we have to
  // pass it the address of ForecastingAgent, needing the other objects created
  // immediately here

  // we create an instance of MatrixMult, VectorTransFunc
  // and of RuleMaster-RuleMaker

  matrixMult = [MatrixMult createBegin: self];
  matrixMult = [matrixMult createEnd];

  transFunc = [TransFunc createBegin: self];
  transFunc = [transFunc createEnd];

  vectorTransFunc = [VectorTransFunc createBegin: self];
  vectorTransFunc = [vectorTransFunc setTransFunc: transFunc];
  vectorTransFunc = [vectorTransFunc createEnd];

  // creating the simpleANNRuleMaker to evolve the ANN owned by the
  // forecastingAgent
  simpleANNRuleMaker=[SimpleANNRuleMaker createBegin: self];
  [simpleANNRuleMaker setMyUniformIntRand: myUniformIntRand];
  [simpleANNRuleMaker setMatrixMult: matrixMult];
  [simpleANNRuleMaker setVectorTransFunc: vectorTransFunc];
  simpleANNRuleMaker=[simpleANNRuleMaker createEnd];

  // creating the simpleANNRuleMaster to apply the ANN owned by the
  // forecastingAgent
  simpleANNRuleMaster=[SimpleANNRuleMaster createBegin: self];
  [simpleANNRuleMaster setSimpleANNRuleMaker: simpleANNRuleMaker];
  [simpleANNRuleMaster setMatrixMult: matrixMult];
  [simpleANNRuleMaster setVectorTransFunc: vectorTransFunc];
  simpleANNRuleMaster=[simpleANNRuleMaster createEnd];

  // creating the forecastingAgent
    if (cleanForecastingANNEveryMgtemNDays<learningProcessEveryNDays ||
        cleanForecastingANNEveryMgtemNDays%learningProcessEveryNDays !=0)
                {printf("cleanForecastingANNEveryMgtemNDays must be\n"
                        "must be greater than or equal and multiple of\n"
                        "learningProcessEveryNDays.\n");
		exit(0);
                                 }

  forecastingAgentArray = [Array create: self];
  [forecastingAgentArray setCount: bookNumber];
  forecastingAgentArrayIndex = [forecastingAgentArray begin: self];


  for (i=1;i<=bookNumber;i++)
  {



   Book * oneBook;
   [bookArrayIndex setOffset: i-1];
   oneBook=[bookArrayIndex get];



  aForecastingAgent=[ForecastingAgent createBegin: self];
  [aForecastingAgent setNumber: i];
  [aForecastingAgent setBook: oneBook];
  [aForecastingAgent setDataWindowLength: dataWindowLength
                andNAheadForecasting: nAheadForecasting
                andForecastingTrainingSetLength: forecastingTrainingSetLength
                andEpochNumberInEachForecastingTrainingCycle:
                   epochNumberInEachForecastingTrainingCycle
                andLearningProcessEveryNDays: learningProcessEveryNDays
                andCleanForecastingANNEveryMgtemNDays:
                                      cleanForecastingANNEveryMgtemNDays
                andForecastHistoryLength: meanPriceHistoryLength];
  [aForecastingAgent setModelSwarmAddress: self];
  [aForecastingAgent setSimpleANNRuleMasterAddress: simpleANNRuleMaster];
  [aForecastingAgent setMyUniformDblRand: myUniformDblRand]; // ad hoc distr.
  [aForecastingAgent setPrinting: printing];
  aForecastingAgent=[aForecastingAgent createEnd];

  [forecastingAgentList addLast: aForecastingAgent];
  [forecastingAgentArrayIndex next];
  [forecastingAgentArrayIndex put: aForecastingAgent];

   ForecastingAgent * oneForecastingAgent;
   [forecastingAgentArrayIndex setOffset: i-1];
   oneForecastingAgent=[forecastingAgentArrayIndex get];

  }

  // aNNForecastAppAgent
  for (i=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber+
         stopLossAgentNumber + 1;
       i<=randomAgentNumber+marketImitatingAgentNumber+locallyImitatingAgentNumber+
          stopLossAgentNumber+aNNForecastAppAgentNumber;i++)
  {
  anAgent5=[ANNForecastAppAgent createBegin: self];
  [anAgent5 setNumber: i];
  [anAgent5 setAsymmetricBuySellProb: asymmetricBuySellProb];
  [anAgent5 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent5 setMaxOrderQuantity2: maxOrderQuantity2];
 // [anAgent5 setBook: theBook];
  [anAgent5 setBookArrayIndex: bookArrayIndex];
  [anAgent5 setBookNumber: bookNumber];

  [anAgent5 setForecastingAgentArrayIndex: forecastingAgentArrayIndex];

  [anAgent5 setANNInactivityRange: aNNInactivityRange
            andANNForecastAppAgentActDailyProb: aNNForecastAppAgentActDailyProb];
  [anAgent5 setMyUniformDblRand: myUniformDblRand]; // ad hoc distr.
  [anAgent5 setRuleMaster: randomRuleMaster];
  //[anAgent5 setForecastingAgent: forecastingAgent];
  [anAgent5 setPrinting: printing];
  anAgent5=[anAgent5 createEnd];

  [agentList addLast: anAgent5];
  [aNNForecastAppAgentList addLast: anAgent5];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent5];
  }

  // BPCT

  // agentForge step 7b
      bPCTAgentAInputNodeNumber  = 7;
      bPCTAgentAHiddenNodeNumber = 5;
      bPCTAgentAOutputNodeNumber = 3;

      bPCTAgentBInputNodeNumber  = 8;
      bPCTAgentBHiddenNodeNumber = 6;
      bPCTAgentBOutputNodeNumber = 5;

      bPCTPatternNumberInVerificationSet = -1;
      bPCTPatternNumberInTrainingSet     = -10;
      bPCTAgentsAreDisplayingData        =  0;
      usingRandomOrderInBPCTLearning     =  1;
      longTermLearningInBPCT_OnlyWithCompleteTrainingSet = 1;
      useOutputsAsTargetsInBPCT_RelearningScheme = 0;
      bPCTWeightRange = 0.3;
      bPCTEps          = 0.6;
      bPCTAlpha        = 0.9;

  [ObjectLoader load: self fromFileNamed: "bp.setup"];

  otherAgents=randomAgentNumber+marketImitatingAgentNumber+
              locallyImitatingAgentNumber+
              stopLossAgentNumber+aNNForecastAppAgentNumber;

  // bPCTAgent A
  // the various rule Master/Maker are private, to keep independent the random
  // distributions
  bPCTRuleMakerA = [BPCTRuleMaker createBegin: self];
  [bPCTRuleMakerA setMyUniformIntRand: myUniformIntRand2]; // ad hoc distr.
  [bPCTRuleMakerA setMatrixMult: matrixMult];
  [bPCTRuleMakerA setVectorTransFunc: vectorTransFunc];
  bPCTRuleMakerA = [bPCTRuleMakerA createEnd];

  bPCTRuleMasterA = [BPCTRuleMaster createBegin: self];
  [bPCTRuleMasterA setRuleMaker: bPCTRuleMakerA];
  [bPCTRuleMasterA setMatrixMult: matrixMult];
  [bPCTRuleMasterA setVectorTransFunc: vectorTransFunc];
  bPCTRuleMasterA = [bPCTRuleMasterA createEnd];

  bPCTPriceRuleMasterA = [BPCTPriceRuleMaster createBegin: self];
  [bPCTPriceRuleMasterA setMyUniformDblRand: myUniformDblRand2]; // ad hoc distr.
  [bPCTPriceRuleMasterA setAgentProbToActBeforeOpening:
                                  agentProbToActBeforeOpening];
  [bPCTPriceRuleMasterA setMinCorrectingCoeff: minCorrectingCoeff];
  [bPCTPriceRuleMasterA setMaxCorrectingCoeff: maxCorrectingCoeff];
  [bPCTPriceRuleMasterA setAsymmetricRange: asymmetricRange];
  bPCTPriceRuleMasterA = [bPCTPriceRuleMasterA createEnd];

  for (i=otherAgents + 1;
       i<=otherAgents + bPCTAgentANumber;i++)
  {
  // we set a random book for the agent

  int randomBookNumber;
  randomBookNumber=[uniformIntRand getIntegerWithMin: 1 withMax: bookNumber];
  Book * oneBook;
  [bookArrayIndex setOffset: randomBookNumber-1];
  oneBook=[bookArrayIndex get];

  // first we create the datawarehouse where BPCTagent technical data are stored
  aDataWarehouse= [BPCTDataWarehouse createBegin: self];
  [aDataWarehouse setMinmaxRowToBeModifiedFromInt: 9
                          usingGenericIntVariableAddress: &maxOrderQuantity];
  [aDataWarehouse setVerificationFileName: unusedFile // never used here
                  andTrainingFileName:     unusedFile // never used here
                  andMinmaxName: bPCTMinmaxFileNameA
                  andInitValuesFileName: bPCTInitValuesFileNameA];
  [aDataWarehouse setInputNodeNumber:  bPCTAgentAInputNodeNumber
                  andHiddenNodeNumber: bPCTAgentAHiddenNodeNumber
                  andOutputNodeNumber: bPCTAgentAOutputNodeNumber
           andPatternNumberInVerificationSet: bPCTPatternNumberInVerificationSet
           andPatternNumberInTrainingSet:     bPCTPatternNumberInTrainingSet
           andEpochNumberInEachTrainingCycle:
                                       epochNumberInEachBPCTTrainingCycle];
  [aDataWarehouse setBackPropagationParametersWeightRange: bPCTWeightRange
                  eps: bPCTEps alpha: bPCTAlpha
                  andWithOrderInLearning: usingRandomOrderInBPCTLearning
                  andLongTermLearningInCT:
                  longTermLearningInBPCT_OnlyWithCompleteTrainingSet
                  andUseOutputsAsTargetsInCT:
                  useOutputsAsTargetsInBPCT_RelearningScheme];
  [aDataWarehouse setMyUniformDblRand: myUniformDblRand2]; // ad hoc dist.

  aDataWarehouse=[aDataWarehouse createEnd];

  // then we create an interface for our agent, to simplifly its links
  // with the observer, if any, but mainly as a help in CT building

  anInterface6 = [BPCTAgentAInterface createBegin: self];
  if (i<=otherAgents+bPCTAgentAEO_EP_0_Number+bPCTAgentAEO_EP_1_Number
                    +bPCTAgentAEO_EP_2_Number+bPCTAgentAEO_EP_3_Number)
    [anInterface6 setUseEO_EP: 3]; // EO_EP 3
  if (i<=otherAgents+bPCTAgentAEO_EP_0_Number+bPCTAgentAEO_EP_1_Number
                    +bPCTAgentAEO_EP_2_Number)
    [anInterface6 setUseEO_EP: 2]; // EO_EP 2
  if (i<=otherAgents+bPCTAgentAEO_EP_0_Number+bPCTAgentAEO_EP_1_Number)
    [anInterface6 setUseEO_EP: 1]; // EO_EP 1
  if (i<=otherAgents+bPCTAgentAEO_EP_0_Number)
    [anInterface6 setUseEO_EP: 0]; // no EO_EP use
  [anInterface6 setEO_EPDelta: agentAEO_EPDelta];
  [anInterface6 setMyUniformIntRand: myUniformIntRand2]; // ad hoc distr.
  [anInterface6 setAgentNumber: i];
  [anInterface6 setDataWarehouse: aDataWarehouse];
  [anInterface6 setBook: oneBook];
  anInterface6 = [anInterface6 createEnd];
  [anInterface6 initialize]; // NB. after createEnd

  anAgent6 = [BPCTAgentA createBegin: self];
  [anAgent6 setNumber: i andSetReadWeightsFromFile: 0]; // it never reads weights
                                                        // from a file
  [anAgent6 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent6 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent6 setDataWarehouse: aDataWarehouse];
  [anAgent6 setInterface: anInterface6];         // double declaration for the parent
  [anAgent6 setSpecificInterface: anInterface6]; // and for the inheriting class
  [anAgent6 setRuleMaster: bPCTRuleMasterA];
  [anAgent6 setPriceRuleMaster: bPCTPriceRuleMasterA];
  [anAgent6 setDisplayDataWhileRunning: bPCTAgentsAreDisplayingData];
  [anAgent6 setBook: oneBook]; // used by accounting method act2
  [anAgent6 setPrinting: printing];

  anAgent6 = [anAgent6 createEnd];


  [agentList addLast: anAgent6];
  [bPCTAgentAList addLast: anAgent6];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent6];

  [bPCTAgentAArrayIndex next];
  [bPCTAgentAArrayIndex put: anAgent6];

  }

  otherAgents+=bPCTAgentANumber;

  // bPCTAgent B
  // the various rule Master/Maker are private, to keep independent the random
  // distributions
  bPCTRuleMakerB = [BPCTRuleMaker createBegin: self];
  [bPCTRuleMakerB setMyUniformIntRand: myUniformIntRand3]; // ad hoc distr.
  [bPCTRuleMakerB setMatrixMult: matrixMult];
  [bPCTRuleMakerB setVectorTransFunc: vectorTransFunc];
  bPCTRuleMakerB = [bPCTRuleMakerB createEnd];

  bPCTRuleMasterB = [BPCTRuleMaster createBegin: self];
  [bPCTRuleMasterB setRuleMaker: bPCTRuleMakerB];
  [bPCTRuleMasterB setMatrixMult: matrixMult];
  [bPCTRuleMasterB setVectorTransFunc: vectorTransFunc];
  bPCTRuleMasterB = [bPCTRuleMasterB createEnd];

  bPCTPriceRuleMasterB = [BPCTPriceRuleMaster createBegin: self];
  [bPCTPriceRuleMasterB setMyUniformDblRand: myUniformDblRand3]; // ad hoc distr.
  [bPCTPriceRuleMasterB setAgentProbToActBeforeOpening:
                                  agentProbToActBeforeOpening];
  [bPCTPriceRuleMasterB setMinCorrectingCoeff: minCorrectingCoeff];
  [bPCTPriceRuleMasterB setMaxCorrectingCoeff: maxCorrectingCoeff];
  [bPCTPriceRuleMasterB setAsymmetricRange: asymmetricRange];
  bPCTPriceRuleMasterB = [bPCTPriceRuleMasterB createEnd];

  for (i=otherAgents + 1;
       i<=otherAgents + bPCTAgentBNumber;i++)
  {
  // first we create the datawarehouse where BPCTagent technical data are stored
  aDataWarehouse= [BPCTDataWarehouse createBegin: self];
  [aDataWarehouse setMinmaxRowToBeModifiedFromInt: 12
                          usingGenericIntVariableAddress: &maxOrderQuantity];
  [aDataWarehouse setVerificationFileName: unusedFile // never used here
                  andTrainingFileName:     unusedFile // never used here
                  andMinmaxName: bPCTMinmaxFileNameB
                  andInitValuesFileName: bPCTInitValuesFileNameB];
  [aDataWarehouse setInputNodeNumber:  bPCTAgentBInputNodeNumber
                  andHiddenNodeNumber: bPCTAgentBHiddenNodeNumber
                  andOutputNodeNumber: bPCTAgentBOutputNodeNumber
           andPatternNumberInVerificationSet: bPCTPatternNumberInVerificationSet
           andPatternNumberInTrainingSet:     bPCTPatternNumberInTrainingSet
           andEpochNumberInEachTrainingCycle:
                                       epochNumberInEachBPCTTrainingCycle];
  [aDataWarehouse setBackPropagationParametersWeightRange: bPCTWeightRange
                  eps: bPCTEps alpha: bPCTAlpha
                  andWithOrderInLearning: usingRandomOrderInBPCTLearning
                  andLongTermLearningInCT:
                  longTermLearningInBPCT_OnlyWithCompleteTrainingSet
                  andUseOutputsAsTargetsInCT:
                  useOutputsAsTargetsInBPCT_RelearningScheme];
  [aDataWarehouse setMyUniformDblRand: myUniformDblRand3]; // ad hoc dist.

  aDataWarehouse=[aDataWarehouse createEnd];

  // then we create an interface for our agent, to simplifly its links
  // with the observer, if any, but mainly as a help in CT building

  anInterface7 = [BPCTAgentBInterface createBegin: self];
  if (i<=otherAgents+bPCTAgentBEO_EP_0_Number+bPCTAgentBEO_EP_1_Number
                    +bPCTAgentBEO_EP_2_Number+bPCTAgentBEO_EP_3_Number)
    [anInterface7 setUseEO_EP: 3]; // EO_EP 3
  if (i<=otherAgents+bPCTAgentBEO_EP_0_Number+bPCTAgentBEO_EP_1_Number
                    +bPCTAgentBEO_EP_2_Number)
    [anInterface7 setUseEO_EP: 2]; // EO_EP 2
  if (i<=otherAgents+bPCTAgentBEO_EP_0_Number+bPCTAgentBEO_EP_1_Number)
    [anInterface7 setUseEO_EP: 1]; // EO_EP 1
  if (i<=otherAgents+bPCTAgentBEO_EP_0_Number)
    [anInterface7 setUseEO_EP: 0]; // no EO_EP use
  [anInterface7 setEO_EPDelta: agentBEO_EPDelta];
  [anInterface7 setMyUniformIntRand: myUniformIntRand3]; // ad hoc distr.
  [anInterface7 setAgentNumber: i];
  [anInterface7 setDataWarehouse: aDataWarehouse];
//  [anInterface7 setBook: theBook];
 // [anInterface7 setForecastingAgent: forecastingAgent];
  anInterface7 = [anInterface7 createEnd];
  [anInterface7 initialize]; // NB. after createEnd

  anAgent7 = [BPCTAgentB createBegin: self];
  [anAgent7 setNumber: i andSetReadWeightsFromFile: 0]; // it never reads weights
                                                        // from a file
  [anAgent7 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent7 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent7 setDataWarehouse: aDataWarehouse];
  [anAgent7 setInterface: anInterface7];         // double declaration for the parent
  [anAgent7 setSpecificInterface: anInterface7]; // and for the inheriting class
  [anAgent7 setRuleMaster: bPCTRuleMasterB];
  [anAgent7 setPriceRuleMaster: bPCTPriceRuleMasterB];
  [anAgent7 setDisplayDataWhileRunning: bPCTAgentsAreDisplayingData];
//  [anAgent7 setBook: theBook]; // used by accounting method act2
  [anAgent7 setPrinting: printing];

  anAgent7 = [anAgent7 createEnd];


  [agentList addLast: anAgent7];
  [bPCTAgentBList addLast: anAgent7];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent7];

  [bPCTAgentBArrayIndex next];
  [bPCTAgentBArrayIndex put: anAgent7];

  }




   otherAgents=randomAgentNumber+marketImitatingAgentNumber+
               locallyImitatingAgentNumber+
              stopLossAgentNumber+aNNForecastAppAgentNumber;

  // AvatarAgent
  otherAgents = otherAgents + bPCTAgentBNumber;

  for (i=otherAgents + 1;
     i<=otherAgents + avatarAgentNumber;i++)
  {
  anAgent8 = [AvatarAgent createBegin: self];
  [anAgent8 setNumber: i];
  [anAgent8 setContinuosMarket: continuosMarket];
  [anAgent8 setBookArrayIndex: bookArrayIndex];
  [anAgent8 setBookNumber: bookNumber];
  [anAgent8 setPrinting: printing];
  [anAgent8 setPenalty: humanPenalty];
  [anAgent8 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent8 setMaxOrderQuantity2: maxOrderQuantity2];
  anAgent8=[anAgent8 createEnd];

  [agentList addLast: anAgent8];
  [avatarAgentList addLast: anAgent8];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent8];
  }

  // ArbitrageurAgent
  otherAgents = otherAgents + avatarAgentNumber;
   for (i= otherAgents + 1;i<=otherAgents + arbitrageurAgentNumber;i++)
  {
  anAgent9 = [ArbitrageurAgent createBegin: self];
  [anAgent9 setNumber: i];
  [anAgent9 setBookArrayIndex: bookArrayIndex];
  [anAgent9 setBookNumber: bookNumber];
  [anAgent9 setArbitrageurOperatingInterval: arbitrageurOperatingInterval];
  [anAgent9 setArbitrageurGain: arbitrageurGain];
  [anAgent9 setArbitrageurOperatingIntervalFixed: arbitrageurOperatingIntervalFixed];
  [anAgent9 setPrinting: printing];
  [anAgent9 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent9 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent9 setIndexCalculator: indexCalculator];
  anAgent9=[anAgent9 createEnd];

  [agentList addLast: anAgent9];
  [arbitrageurAgentList addLast: anAgent9];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent9];
  }


 // eventAgent
  otherAgents = otherAgents + arbitrageurAgentNumber;
  for (i=otherAgents+1;i<=otherAgents+eventAgentNumber;i++)
  {
  anAgent10=[EventAgent createBegin: self];
  [anAgent10 setNumber: i];
  [anAgent10 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent10 setMaxOrderQuantity2: maxOrderQuantity2];
  [anAgent10 setBookArrayIndex: bookArrayIndex];
  [anAgent10 setBookNumber: bookNumber];
  [anAgent10 setEventGenerator: eventGenerator];
  [anAgent10 setRuleMaster: eventRuleMaster];
  [anAgent10 setPrinting: printing];
  anAgent10=[anAgent10 createEnd];

  [agentList addLast: anAgent10];
  [eventAgentList addLast: anAgent10];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent10];
  }

// wasteTimeAgent
  otherAgents = otherAgents + eventAgentNumber;
  for (i=otherAgents+1;i<=otherAgents+wasteTimeAgentNumber;i++)
  {

  anAgent11=[WasteTimeAgent createBegin: self];
  [anAgent11 setNumber: i];
  [anAgent11 setDelay: delay];
  [anAgent11 setMaxOrderQuantity: maxOrderQuantity];
  [anAgent11 setMaxOrderQuantity: maxOrderQuantity2];
  [anAgent11 setPrinting: printing];
  anAgent11=[anAgent11 createEnd];

  [agentList addLast: anAgent11];
  [wasteTimeAgentList addLast: anAgent11];

  [agentArrayIndex next];
  [agentArrayIndex put: anAgent11];
  }

  // agentForge step 8






  // current agent
  // see the comment in CurrentAgent.m to understand this trick
  theCurrentAgent=[CurrentAgent createBegin: self];
  [theCurrentAgent setAgentList: agentList];
  theCurrentAgent=[theCurrentAgent createEnd];

  return self;
}

- buildActions
{
  int i;
  // we create the list of simulation actions
  modelActions1 = [ActionGroup create: self];
  [modelActions1 createActionTo: self message: M(setDayStateOpeningAuction)];
  [modelActions1 createActionTo: self message: M(increaseCurrentDayNumber)];
  [modelActions1 createActionForEach: bookList message: M(setClean)];
  [modelActions1 createActionForEach: forecastingAgentList message: M(step)];
  if(humanPenalty==2)  [modelActions1 createActionTo: self message: M(check24hSpent)];
  //if((humanPenalty==2)&&(newDay==1)&&(avatarAgentNumber>0))[modelActions1 createActionForEach: avatarAgentList message: M(onlineExperimentPenaltyAccounting)];
  // Prepare Agent matrix
  [modelActions1 createActionForEach: agentList message: M(actPrepare)];
  //  [modelActions1 createActionTo: quota message: M(step)];
  // this shuffling action is not relevant if agent are equal instances of
  // the same class, but it is useful to avoid biased situations when agents
  // are not homegeneous
  [modelActions1 createActionTo: listShuffler
                             message: M(shuffleWholeList:) : agentList];
  // acting before opening
  [modelActions1 createActionTo: eventGenerator message: M(setEventState)];
  [modelActions1 createActionForEach: agentList message: M(act0)];
  [modelActions1 createActionTo: self message: M(waitForOrders)];
  if (avatarAgentNumber>0)[modelActions1 createActionForEach: avatarAgentList message: M(act0)];
  if (arbitrageurAgentNumber>0)[modelActions1 createActionForEach: arbitrageurAgentList message: M(act0)];
  [modelActions1 createActionForEach: bookList message: M(openingAuction)];
  [modelActions1 createActionTo: self message: M(setDayStateContinuousTrading)];

  // acting in the market
  //status=2;
  modelActions2 = [ActionGroup create: self];
  [modelActions2 createActionTo: eventGenerator message: M(setEventState)];
  [modelActions2 createActionTo: theCurrentAgent message: M(act1)];
  if (arbitrageurAgentNumber>0)[modelActions2 createActionForEach: arbitrageurAgentList message: M(act1)];
  if (avatarAgentNumber>0)[modelActions2 createActionForEach: avatarAgentList message: M(act1)];
  if (arbitrageurAgentNumber>0)[modelActions2 createActionForEach: arbitrageurAgentList message: M(act1)];

  // actiong in closing
  modelActions3 = [ActionGroup create: self];
  [modelActions3 createActionTo: self message: M(setDayStateClosingAuction)];
  [modelActions3 createActionTo: eventGenerator message: M(setEventState)];
  [modelActions3 createActionForEach: agentList message: M(act0)];
  [modelActions3 createActionTo: self message: M(waitForOrders)];
  if (avatarAgentNumber>0)[modelActions3 createActionForEach: avatarAgentList message: M(act0)];
  if (arbitrageurAgentNumber>0)[modelActions3 createActionForEach: arbitrageurAgentList message: M(act0)];
  [modelActions3 createActionForEach: bookList message: M(closingAuction)];


  // accounting ...
  //status=1;
  modelActions4 = [ActionGroup create: self];
  [modelActions4 createActionTo: self message: M(setDayStateOpeningAuction)];
  [modelActions4 createActionForEach: bookList message: M(setMeanPrice)];
  [modelActions4 createActionForEach: agentList message: M(act2)];
  [modelActions4 createActionTo: indexCalculator message: M(ohlc)];
  // then we create a schedule that executes the modelActions.

  modelSchedule = [Schedule createBegin: self];
  // we use here agentNumber steps in each cycle, while the observer uses
  // a low display frequency (e.g. 1) to show the price of each step-tick
  // we can also use a high d.f. (e.g.1000 with 100 agents) to run a faster
  // simulation
  [modelSchedule setRepeatInterval: agentNumber+1];
  modelSchedule = [modelSchedule createEnd];

  [modelSchedule at: 0 createAction: modelActions1];

  for (i=0;i<agentNumber;i++){
  	[modelSchedule at: i createAction: modelActions2];
  }
  [modelSchedule at: agentNumber-1 createAction: modelActions3];
  [modelSchedule at: agentNumber   createAction: modelActions4];

  return self;
}

- activateIn: swarmContext
{
  // here, we activate the swarm in the context passed in
  // then we activate our schedule in ourselves

  [super activateIn: swarmContext];

  [modelSchedule activateIn: self];

  return [self getSwarmActivity];
}

- increaseCurrentDayNumber
{
  time_t t;
  int timestamp;
  timestamp=time (&t);

  dayNumber++;
  if(printing==1)printf("Day number #%5d\n",dayNumber);
	FILE * pFile;
	pFile = fopen ("Day.dat","w");
	if (pFile == NULL) {
	  perror("cannot open output file model");
	  exit(1);
	}else
	fprintf(pFile,"%5d\n", dayNumber);
	fclose(pFile);


	FILE * ppFile;
	ppFile = fopen ("dayhistory.dat","a");
	if (ppFile == NULL) {
	  perror("cannot open output file model");
	  exit(1);
	}
	fprintf(ppFile,"%5d\t%20d\n", dayNumber, timestamp);
	fclose(ppFile);
  return self;
}

- (int) getCurrentDay
{
  return dayNumber;
}

-  check24hSpent
{
	time_t t;
	int timestamp,oldTimestamp;
	timestamp=time (&t);

	FILE * pFile;
	pFile = fopen ("day.timestamp","r");
	if (pFile == NULL) {

		pFile = fopen ("day.timestamp","w");
		fprintf(pFile,"%20d\n", timestamp);
		fclose(pFile);
		return self;
	}else{
      fscanf(pFile,"%d\n", &oldTimestamp);
  	  fclose(pFile);

	if (timestamp>=oldTimestamp+86400){
		[avatarAgentList  forEach:M(onlineExperimentPenaltyAccounting)];
		if(printing==1)printf("A New Day \n");

		FILE * ppFile;
		ppFile = fopen ("dayhistory.dat","a");
		if (ppFile == NULL) {
		  perror("cannot open output file model");
		  exit(1);
		}
		fprintf(ppFile,"%5d\t -NewDay-\n", dayNumber);
		fclose(ppFile);

		pFile = fopen ("day.timestamp","w");
		fprintf(pFile,"%20d\n", timestamp);
		fclose(pFile);
		return self;
	}
	}

  return self;
}



  // agentForge step 9
- getAgentList{return agentList;}
- getRandomAgentList{return randomAgentList;}
- getArbitrageurAgentList{return arbitrageurAgentList;}
- getWasteTimeAgentList{return wasteTimeAgentList;}
- getAgentArrayIndex{return agentArrayIndex;}
- getBookArrayIndex{return bookArrayIndex;}

- (int) getBPCTAgentANumber{return bPCTAgentANumber;}
- (int) getBPCTAgentBNumber{return bPCTAgentBNumber;}

- getAvatarAgentList{return avatarAgentList;}
- getEventAgentList{return eventAgentList;}
- getMarketImitatingAgentList{return marketImitatingAgentList;}
- getLocallyImitatingAgentList{return locallyImitatingAgentList;}
- getStopLossAgentList{return stopLossAgentList;}
- getANNForecastAppAgentList{return aNNForecastAppAgentList;}
- getBPCTAgentAList{return bPCTAgentAList;}
- getBPCTAgentBList{return bPCTAgentBList;}

- getBPCTAgentAArrayIndex{return bPCTAgentAArrayIndex;}
- getBPCTAgentBArrayIndex{return bPCTAgentBArrayIndex;}


// - getForecastingAgentArrayIndex{return forecastingArrayIndex;}

- getIndexCalculator{return indexCalculator;}

- getForecastingAgent: (int) n
{
  ForecastingAgent * oneForecastingAgent;
  [forecastingAgentArrayIndex setOffset: n-1];
  oneForecastingAgent=[forecastingAgentArrayIndex get];
  return oneForecastingAgent;
}

- getEventGenerator
{
  return eventGenerator;
}


- getBook: (int) n
{
  Book * oneBook;
  [bookArrayIndex setOffset: n-1];
  oneBook=[bookArrayIndex get];
  return oneBook;
}


- setDayStateOpeningAuction
{
  FILE * pFile;
  pFile = fopen ("daystate.dat","w");
  if (pFile == NULL) {
	perror("cannot open output file model");
 	exit(1);
	}
  fprintf(pFile,"Opening Auction");
  fclose(pFile);
  return self;
}

- setDayStateContinuousTrading
{
  FILE * pFile;
  pFile = fopen ("daystate.dat","w");
  if (pFile == NULL) {
	perror("cannot open output file model");
	exit(1);
	}
  fprintf(pFile,"Continuous Trading");
  fclose(pFile);
  return self;
}

- setDayStateClosingAuction
{
  FILE * pFile;
  pFile = fopen ("daystate.dat","w");
  if (pFile == NULL) {
	perror("cannot open output file model");
 	exit(1);
	}
  fprintf(pFile,"Closing Auction");
  fclose(pFile);
  return self;
}


- waitForOrders
{
   sleep(delayInAuction);

   return self;
}

- (int) getBookNumber
{
  return bookNumber;
}

- openProbeTo: (int) n
{
  BasicSumAgent * anAgent;

  if (n<1 || n>agentNumber) return self;

  [agentArrayIndex setOffset: n-1];
  anAgent=[agentArrayIndex get];
  [anAgent getProbe];

  return anAgent;
}


@end









































