// ObserverSwarm.m

#import "ObserverSwarm.h"
#import <activity.h>
#import <simtoolsgui.h>

@implementation ObserverSwarm

+ createBegin: aZone
{
  ObserverSwarm *obj;
  id <ProbeMap> probeMap;

  // Superclass createBegin to allocate ourselves.

  obj = [super createBegin: aZone];

  // Fill in the relevant parameters.

  obj->displayFrequency          = 487; // AgentNumber  +1

  obj->stopAtDayNumber           = 20; // to stop the program (if != 0)
  obj->displayPreviousDayMean    = 0;
  obj->showPriceGraph            = 0;
  obj->savePriceData             = 1;
  obj->showBookGraph             = 0;
  obj->saveBookData              = 1;
  obj->showVolumesGraph          = 0;
  obj->saveVolumesData           = 1;
  obj->showAgentWealthGraph      = 0;
  obj->saveAgentWealthData       = 1;
  obj->showForecastingAgentGraph = 0;
  obj->saveForecastingData       = 1;
  obj->showEventsGraph	         = 0;
  obj->saveEventsData            = 1;

  //agentForge step 11b
  obj->numberOfTheBPCTAgentA_ToBeObservedDirectly = 0;
  obj->saveBPCTAgentAData = 0;
  obj->numberOfTheBPCTAgentB_ToBeObservedDirectly = 0;
  obj->saveBPCTAgentBData = 0;

  // to build a customized probe map
  // without a probe map, the default is to show all variables and messages
  // here we choose to customize the appearance of the probe, to give a nicer
  // interface

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // add variables to be probed

  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "displayFrequency"           inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "stopAtDayNumber"            inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "displayPreviousDayMean"     inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showPriceGraph"             inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "savePriceData"              inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showBookGraph"              inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBookData"               inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showVolumesGraph"           inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveVolumesData"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showAgentWealthGraph"       inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveAgentWealthData"        inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showForecastingAgentGraph"  inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveForecastingData"        inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "showEventsGraph"            inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveEventsData"             inClass: [self class]]];


  //agentForge step 11c
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "numberOfTheBPCTAgentA_ToBeObservedDirectly"
                                         inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBPCTAgentAData"         inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "numberOfTheBPCTAgentB_ToBeObservedDirectly"
                                         inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable:
            "saveBPCTAgentBData"         inClass: [self class]]];

  // set our custom probeMap into the probeLibrary as the default probe for
  // the ObserverSwarm class

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

- createEnd
{
  return [super createEnd];
}


- buildObjects
{
  int i;
  [super buildObjects];

  modelSwarm = [ModelSwarm create: self];

  // to create probe objects on the model and the observer (self, here)
  // ARCHIVED to allow the "Save" button to operate

  CREATE_ARCHIVED_PROBE_DISPLAY (modelSwarm);
  CREATE_ARCHIVED_PROBE_DISPLAY (self);

  // we pause here to allow the parameters to be changed.

  //[controlPanel setStateStopped];
  //2002-10-01 for a continous running decomment the line below, and comment the line above
  [controlPanel setStateRunning];


  // The system will wait until the user hits "Start" or "Next"
  // on the control panel

  [modelSwarm buildObjects];

  // checking the consistence of the displayFrequency with the agentNumber:
  // for display reasons it is necessary to update the graphic widgets
  // after all agent actions (mainly the BPCT ones, that calculate their
  // targets at the end of a day and their inputs at the beginning of the day)
  // so the displays are updated at displayFrequency-1
  // if displayFrequency=1, in the agentNumber steps of a day we have the
  // new BPCT input and output updated at the first step and the BPCT target
  // updated at the last
  // if we adopt a displayFrequency multiple of angentNumber we have the
  // apparent coincidence of these values (##)
/*
  if(displayFrequency !=1 && displayFrequency %
     [[modelSwarm getAgentList] getCount] != 0)
    {
      printf("displayFrequency must be 1 or multiple of agentNumber.\n");
      exit(0);
    }
*/
//van  theBook=[modelSwarm getBook];
//van  forecastingAgent=[modelSwarm getForecastingAgent];

indexCalculator=[modelSwarm getIndexCalculator];
  eventGenerator=[modelSwarm getEventGenerator];
  // Observer display objects.

  // The current price graph
  priceGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (priceGraph); // to allow "Save"

  if(savePriceData==1)[priceGraph setFileOutput: (BOOL) 1];
                      // to send the data also to a file
  if(showPriceGraph==0)[priceGraph setGraphics: (BOOL) 0];

  [priceGraph setTitle: "Current price"];
  [priceGraph setAxisLabelsX: "Ticks x days." Y: "Price"];
  priceGraph = [priceGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

bookNumber=[modelSwarm getBookNumber];

for (i=1;i<=bookNumber;i++)
{
  oneBook=[modelSwarm getBook: i];
  char bookname [30];
  sprintf(bookname,"Current price book %d",i);
  [priceGraph createSequence: bookname withFeedFrom: oneBook
                                              andSelector: M(getPrice)];
  if(displayPreviousDayMean==1){
  char bookname2 [30];
  sprintf(bookname2,"Day-1 mean p. book %d",i);
  [priceGraph createSequence: bookname2 withFeedFrom: oneBook
                                              andSelector: M(getMeanPrice)];
}}

  [priceGraph createSequence: "Index Value" withFeedFrom: indexCalculator
                                              andSelector: M(getIndexValue)];

  // The book graph


  bookGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (bookGraph); // to allow "Save"

  if(saveBookData==1)[bookGraph setFileOutput: (BOOL) 1];
                     // to send the data also to a file
  if(showBookGraph==0)[bookGraph setGraphics: (BOOL) 0];

  [bookGraph setTitle: "Book log"];
  [bookGraph setAxisLabelsX: "Ticks x days." Y: "Sell and Buy Orders in Log."];
  bookGraph = [bookGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

for (i=1;i<=bookNumber;i++)
{
  oneBook=[modelSwarm getBook: i];
  char bookname [30];
  sprintf(bookname,"Sell orders book %d",i);

  [bookGraph createSequence: bookname withFeedFrom: oneBook
                                        andSelector: M(getSellOrderNumber)];
  char bookname2 [30];
  sprintf(bookname2,"Buy orders book %d",i);
  [bookGraph createSequence: bookname2  withFeedFrom: oneBook
                                        andSelector: M(getBuyOrderNumber)];
}


  // The Events graph

  eventsGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (eventsGraph); // to allow "Save"

  if(saveEventsData==1)[eventsGraph setFileOutput: (BOOL) 1];
                     // to send the data also to a file
  if(showEventsGraph==0)[eventsGraph setGraphics: (BOOL) 0];

  [eventsGraph setTitle: "Events log"];
  [eventsGraph setAxisLabelsX: "Ticks x days." Y: "Events intensity in Log."];
  eventsGraph = [eventsGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

  [eventsGraph createSequence: "Events" withFeedFrom: eventGenerator
                                        andSelector: M(getEventState)];



  // The Volumes graph


  volumesGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (volumesGraph); // to allow "Save"

  if(saveVolumesData==1)[volumesGraph setFileOutput: (BOOL) 1];
                     // to send the data also to a file
  if(showVolumesGraph==0)[volumesGraph setGraphics: (BOOL) 0];

  [volumesGraph setTitle: "Volumes log"];
  [volumesGraph setAxisLabelsX: "Ticks x days." Y: "Volumes in Log."];
  volumesGraph = [volumesGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd


for (i=1;i<=bookNumber;i++)
{
  oneBook=[modelSwarm getBook: i];
  char bookname [30];
  sprintf(bookname,"Price Volumes book %d",i);

  [volumesGraph createSequence: bookname withFeedFrom: oneBook
                                        andSelector: M(getPriceVolumes)];
  char bookname2 [30];
  sprintf(bookname2,"Qty Volumes book %d",i);
  [volumesGraph createSequence: bookname2  withFeedFrom: oneBook
                                        andSelector: M(getQuantityVolumes)];
}



  // The agent's wealth graph


  agentWealthGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (agentWealthGraph); // to allow "Save"

  if(showAgentWealthGraph==0)[agentWealthGraph setGraphics: (BOOL) 0];

  if(saveAgentWealthData==1)[agentWealthGraph setFileOutput: (BOOL) 1];
                      // to send the data also to a file

  [agentWealthGraph setTitle: "Agent's wealth"];
  [agentWealthGraph setAxisLabelsX: "Ticks x days."
                                     Y: "Wealth."];
  agentWealthGraph = [agentWealthGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

  [agentWealthGraph createMinSequence: "MinWealth (all)"
                    withFeedFrom: [modelSwarm getAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (all)"
                    withFeedFrom: [modelSwarm getAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (all)"
                    withFeedFrom: [modelSwarm getAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (r.)"
                    withFeedFrom: [modelSwarm getRandomAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (r.)"
                    withFeedFrom: [modelSwarm getRandomAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (r.)"
                    withFeedFrom: [modelSwarm getRandomAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
                    //nuovo
  [agentWealthGraph createMinSequence: "MinWealth (av.)"
                    withFeedFrom: [modelSwarm getAvatarAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (av.)"
                    withFeedFrom: [modelSwarm getAvatarAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (av.)"
                    withFeedFrom: [modelSwarm getAvatarAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
                    //fine nuovo

  [agentWealthGraph createMinSequence: "MinWealth (arb.)"
                    withFeedFrom: [modelSwarm getArbitrageurAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (arb.)"
                    withFeedFrom: [modelSwarm getArbitrageurAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (arb.)"
                    withFeedFrom: [modelSwarm getArbitrageurAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (e.)"
                    withFeedFrom: [modelSwarm getEventAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (e.)"
                    withFeedFrom: [modelSwarm getEventAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (e.)"
                    withFeedFrom: [modelSwarm getEventAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];



  [agentWealthGraph createMinSequence: "MinWealth (m.i.)"
                    withFeedFrom: [modelSwarm getMarketImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (m.i.)"
                    withFeedFrom: [modelSwarm getMarketImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (m.i.)"
                    withFeedFrom: [modelSwarm getMarketImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (l.i.)"
                    withFeedFrom: [modelSwarm getLocallyImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (l.i.)"
                    withFeedFrom: [modelSwarm getLocallyImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (l.i.)"
                    withFeedFrom: [modelSwarm getLocallyImitatingAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (s.l.)"
                    withFeedFrom: [modelSwarm getStopLossAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (s.l.)"
                    withFeedFrom: [modelSwarm getStopLossAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (s.l.)"
                    withFeedFrom: [modelSwarm getStopLossAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMinSequence: "MinWealth (ann)"
                    withFeedFrom: [modelSwarm getANNForecastAppAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (ann)"
                    withFeedFrom: [modelSwarm getANNForecastAppAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (ann)"
                    withFeedFrom: [modelSwarm getANNForecastAppAgentList]
                    andSelector: M(getWealthAtMeanDailyPrice)];

  [agentWealthGraph createMinSequence: "MinWealth (bPCTA)"
                    withFeedFrom: [modelSwarm getBPCTAgentAList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (bPCTA)"
                    withFeedFrom: [modelSwarm getBPCTAgentAList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (bPCTA)"
                    withFeedFrom: [modelSwarm getBPCTAgentAList]
                    andSelector: M(getWealthAtMeanDailyPrice)];

  [agentWealthGraph createMinSequence: "MinWealth (bPCTB)"
                    withFeedFrom: [modelSwarm getBPCTAgentBList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createAverageSequence: "MeanWealth (bPCTB)"
                    withFeedFrom: [modelSwarm getBPCTAgentBList]
                    andSelector: M(getWealthAtMeanDailyPrice)];
  [agentWealthGraph createMaxSequence: "MaxWealth (bPCTB)"
                    withFeedFrom: [modelSwarm getBPCTAgentBList]
                    andSelector: M(getWealthAtMeanDailyPrice)];



  //agentForge step 11d (before this point)

  // The forecastingAgent graph


  forecastingAgentGraph = [EZGraph createBegin: self];
  SET_WINDOW_GEOMETRY_RECORD_NAME (forecastingAgentGraph); // to allow "Save"

  if(saveForecastingData==1)[forecastingAgentGraph setFileOutput: (BOOL) 1];
                                           // to send the data also to a file
  if(showForecastingAgentGraph==0)[forecastingAgentGraph setGraphics: (BOOL) 0];

  [forecastingAgentGraph setTitle: "Forecasting agent log"];
  [forecastingAgentGraph setAxisLabelsX:
                         "Ticks x days." Y: "Estimates and errors."];
  forecastingAgentGraph = [forecastingAgentGraph createEnd];
  // that above is the old way of creating an EZgraph; now a unique
  // methos exists

  // pay attention: the lines below cannot be placed before createEnd

for (i=1;i<=bookNumber;i++)
{
  oneBook=[modelSwarm getBook: i];
  oneForecastingAgent=[modelSwarm getForecastingAgent: i];
  char bookname [30];
  sprintf(bookname,"Day-1 mean p. (index) book %d",i);

  [forecastingAgentGraph createSequence:  bookname
			 withFeedFrom: oneBook
                         andSelector: M(getMeanPriceIndex)];
  char bookname1 [30];
  sprintf(bookname1,"Day-1 estimate (index) book %d",i);

  [forecastingAgentGraph createSequence: bookname1
			 withFeedFrom: oneForecastingAgent
                         andSelector: M(getPastEstimateIndex)];
  char bookname2 [30];
  sprintf(bookname2,"Back propagation error book %d",i);

  [forecastingAgentGraph createSequence: bookname2
            withFeedFrom: oneForecastingAgent
            andSelector: M(getBackPropagationErrorInTrainingSet)];
  char bookname3 [30];
  sprintf(bookname3,"Proportional error book %d",i);

  [forecastingAgentGraph createSequence: bookname3
            withFeedFrom: oneForecastingAgent
            andSelector: M(getProportionalErrorInTrainingSet)];
}



  // --- BPCT ---

  // agent A

  if([modelSwarm getBPCTAgentANumber] == 0)
                         numberOfTheBPCTAgentA_ToBeObservedDirectly=0;

  // this may be 0 if we do not have any agent (above) or if we choose to
  // have no display
  if (numberOfTheBPCTAgentA_ToBeObservedDirectly > 0)
                                // the value comes from probe
  {
  // to identify the address of the chosen agent

  [[modelSwarm getBPCTAgentAArrayIndex] setOffset:
                                numberOfTheBPCTAgentA_ToBeObservedDirectly-1];
  agentA = [[modelSwarm getBPCTAgentAArrayIndex] get];
  agentAInterface = [agentA getInterface];

   // agent A graph
   bPCTAgentA_Graph = [EZGraph createBegin: [self getZone]];
   SET_WINDOW_GEOMETRY_RECORD_NAME (bPCTAgentA_Graph); // to allow "Save"
   [bPCTAgentA_Graph setTitle: "BPCTAgent A data"];
   [bPCTAgentA_Graph setAxisLabelsX:
    "Ticks x days" Y: "Value"];
   bPCTAgentA_Graph = [bPCTAgentA_Graph createEnd];

   if(saveBPCTAgentAData==1)[bPCTAgentA_Graph setFileOutput: (BOOL) 1];
                                 // to send the data also to a file

   [bPCTAgentA_Graph createSequence: "meanPrice1_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getMeanPrice1)];

   [bPCTAgentA_Graph createSequence: "liquidityQuantity_out_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getLiquidityQuantity_out)];
   [bPCTAgentA_Graph createSequence: "liquidityQuantity_target_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getLiquidityQuantity_target)];
   [bPCTAgentA_Graph createSequence: "shareQuantity_out_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getShareQuantity_out)];
   [bPCTAgentA_Graph createSequence: "shareQuantity_target_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getShareQuantity_target)];
   [bPCTAgentA_Graph createSequence: "buySell_out_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getBuySell_out)];
   [bPCTAgentA_Graph createSequence: "buySell_target_A"
                                 withFeedFrom: agentAInterface
                                 andSelector: M(getBuySell_target)];

  }

  // agent B

  if([modelSwarm getBPCTAgentBNumber] == 0)
                         numberOfTheBPCTAgentB_ToBeObservedDirectly=0;

  // this may be 0 if we do not have any agent (above) or if we choose to
  // have no display
  if (numberOfTheBPCTAgentB_ToBeObservedDirectly > 0)
                                // the value comes from probe
  {
  // to identify the address of the chosen agent

  [[modelSwarm getBPCTAgentBArrayIndex] setOffset:
                                numberOfTheBPCTAgentB_ToBeObservedDirectly-1];
  agentB = [[modelSwarm getBPCTAgentBArrayIndex] get];
  agentBInterface = [agentB getInterface];

   // agent B graph
   bPCTAgentB_Graph = [EZGraph createBegin: [self getZone]];
   SET_WINDOW_GEOMETRY_RECORD_NAME (bPCTAgentB_Graph); // to allow "Save"
   [bPCTAgentB_Graph setTitle: "BPCTAgent B data"];
   [bPCTAgentB_Graph setAxisLabelsX:
    "Ticks x days" Y: "Value"];
   bPCTAgentB_Graph = [bPCTAgentB_Graph createEnd];

   if(saveBPCTAgentBData==1)[bPCTAgentB_Graph setFileOutput: (BOOL) 1];
                                 // to send the data also to a file

   [bPCTAgentB_Graph createSequence: "meanPrice1_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getMeanPrice1)];

   [bPCTAgentB_Graph createSequence: "liquidityQuantity_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getLiquidityQuantity_out)];
   [bPCTAgentB_Graph createSequence: "liquidityQuantity_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getLiquidityQuantity_target)];
   [bPCTAgentB_Graph createSequence: "shareQuantity_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getShareQuantity_out)];
   [bPCTAgentB_Graph createSequence: "shareQuantity_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getShareQuantity_target)];
   [bPCTAgentB_Graph createSequence: "closingPriceWealth_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getClosingPriceWealth_out)];
   [bPCTAgentB_Graph createSequence: "closingPriceWealth_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getClosingPriceWealth_target)];
   [bPCTAgentB_Graph createSequence: "forecastedPriceWealth_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getForecastedPriceWealth_out)];
   [bPCTAgentB_Graph createSequence: "forecastedPriceWealth_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getForecastedPriceWealth_target)];
  [bPCTAgentB_Graph createSequence: "buySell_out_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getBuySell_out)];
   [bPCTAgentB_Graph createSequence: "buySell_target_B"
                                 withFeedFrom: agentBInterface
                                 andSelector: M(getBuySell_target)];

  }


  // agentForge step 11 (before this point, add ...)

  // to check (pressing only once the next or the start button) the initial
  //settings, uncomment the following line
  //  [controlPanel setStateStopped];

  return self;
}

- buildActions
{
  [super buildActions];

  // model schedule.

  [modelSwarm buildActions];

  //agentForge step 11e
  bPCTPatternNumberInVerificationSet=0;
  if(agentA != nil)bPCTPatternNumberInVerificationSet=
                             [agentA getBPCTPatternNumberInVerificationSet];
  if(agentB != nil)bPCTPatternNumberInVerificationSet=
                             [agentB getBPCTPatternNumberInVerificationSet];
  // this variable is used below for a consistence check, as we need
  // here to have a verification set on length 1 (the same valued is used also
  // by the other BPCT agent; we have to repeat the assign operation in case
  // of missing categories of agents)

  // ActionGroup for Observer display

  displayActions = [ActionGroup create: self];

  // to update probes
  [displayActions createActionTo: probeDisplayManager message: M(update)];
  [displayActions createActionTo: actionCache message: M(doTkEvents)];

  [displayActions createActionTo: priceGraph message: M(step)];
   if (showBookGraph==1)
   [displayActions createActionTo: bookGraph  message: M(step)];
   if (showVolumesGraph==1)
   [displayActions createActionTo: volumesGraph  message: M(step)];
  if (showAgentWealthGraph==1)
  [displayActions createActionTo: agentWealthGraph  message: M(step)];
  if (showEventsGraph==1)
  [displayActions createActionTo: eventsGraph message: M(step)];
   if (showForecastingAgentGraph==1)
   [displayActions createActionTo: forecastingAgentGraph  message: M(step)];

  [displayActions createActionTo: self message: M(checkToStop)];

  // Display schedule. Note that the repeat interval is set by our
  // own Swarm data structure. The display is frequently the slowest part of a
  // simulation, when it is redraw less frequently things are faster

  displaySchedule = [Schedule createBegin: self];
  // we can use here a display frequency lower (e.g.1) than the number of steps
  // in the model (step number = agentNumber) to display the prices of each
  // step-tick
  [displaySchedule setRepeatInterval: displayFrequency];
  displaySchedule = [displaySchedule createEnd];

  [displaySchedule at: displayFrequency-1 createAction: displayActions];
                                              // see ## comment above

  //agentForge step 11f
  // BPCT

  if (numberOfTheBPCTAgentA_ToBeObservedDirectly > 0)
  // the following if condition is related to this CT use of the program, to
  // avoid nonsens
   if (bPCTPatternNumberInVerificationSet == -1)
     [displaySchedule at: displayFrequency-1  // see ## comment above
                        createActionTo: bPCTAgentA_Graph message: M(step)];

  if (numberOfTheBPCTAgentB_ToBeObservedDirectly > 0)
  // the following if condition is related to this CT use of the program, to
  // avoid nonsens
   if (bPCTPatternNumberInVerificationSet == -1)
     [displaySchedule at: displayFrequency-1  // see ## comment above
                        createActionTo: bPCTAgentB_Graph message: M(step)];

  return self;
}

- activateIn: swarmContext
{
// activateIn: - to activate the schedules to make them ready to run.

  [super activateIn: swarmContext];

  [modelSwarm activateIn: self];

  [displaySchedule activateIn: self];

  return [self getSwarmActivity];
}

  // to check for the stopping conditions
- checkToStop
{

  // if stopAtDayNumber is left to 0, the program will never stop
  // this stop occurs after the first tick of the time in modelSwarm,
  // but this fact does not affect the results in the observerSwarm
  // being the stop performed before the graph update

  if (stopAtDayNumber !=0 &&
      stopAtDayNumber <= [modelSwarm getCurrentDay])
  {
      printf("Stopping at day number %4d\n",
                                [modelSwarm getCurrentDay]);

      // we can restart by pressing "Start" or "Next", but the simulation
      // will run for displayFrequency ticks and then it will stop again

      // [controlPanel setStateStopped];
      // 2002-10-01 for a continous running decomment the line below, and comment the line above
       [controlPanel setStateQuit];

  }else{
          printf("Day number %4d is finished\n",
                                  [modelSwarm getCurrentDay]);
  }

  return self;
}


@end


