// **********************************
//  This file is derived - with a lot of changes - from bp-ct package
// **********************************

// SimpleANNRuleMaker.m                                        

#import "SimpleANNRuleMaker.h"
#import <math.h>
#import <stdlib.h> // to use calloc()

@implementation SimpleANNRuleMaker

- createEnd
{
   [super createEnd];

//  CREATE_ARCHIVED_PROBE_DISPLAY (self);
  // while debugging, right click on a matrix2 label, open probe (newly
  // right clicking on the box Matrix2) and execute 'print': the matrix
  // content will appear in the terminal

  // initial pointer to internal vector p, used to randomize
  // the learning sequence

   p=NULL;  // if p==NULL the vector has not been yet allocated, see below

   return self;                        
}

- setMyUniformIntRand: u
{
  myUniformIntRand = u;
  return self;
}

- setMatrixMult: mM
{
  matrixMult=mM;
  return self;
}

- setVectorTransFunc: vts
{
  vectorTransFunc=vts;
  return self;
}

- adaptRulesToInputOutputValuesInForecastingTrainingSet: forecastingAg;

  // we are here working with a neural network y=f(B f(Ax))
  // where x and y are vectors, A and B are matrixes and f
  // is a tranformation function, here the
  // logistic one r(s)=(1+e^(-k*s))^-1

  // when introducing a constant value (the bias) as input and hidden node
  // (neuron)
  //            y=f(B f(Ax)) becomes
  //            y=f(B (1, f(A (1,x')')' )' )

{
   int i, I, j, k, iii, n, temp, row, col, colOut, epochNumber;
   float sum, delta;

  forecastingAgent=forecastingAg;

  // getting parameters and addresses of the matrixes

  forecastingTrainingSet   = [forecastingAgent getForecastingTrainingSet];
  outputTrainingMatrix = [forecastingAgent getOutputTrainingMatrix];
  targetTrainingMatrix = [forecastingAgent getTargetTrainingMatrix];
  minmax=[forecastingAgent getMinmax];
  inputNodeNumber  = [forecastingAgent getDataWindowLength];
  inputNodeNumber1 = inputNodeNumber+1;
  hiddenNodeNumber = [forecastingAgent getHiddenNodeNumber];
  hiddenNodeNumber1= hiddenNodeNumber+1;
  outputNodeNumber = 1;
  patternNumberInTrainingSet = [forecastingAgent
                                         getForecastingTrainingSetLength];
  epochNumberInEachTrainingCycle  = [forecastingAgent
                                getEpochNumberInEachForecastingTrainingCycle];

  wih = [forecastingAgent getWih];
  dwih= [forecastingAgent getDwih];
  who = [forecastingAgent getWho];
  dwho= [forecastingAgent getDwho];
  inputLayer  = [forecastingAgent getInputLayer];
  hiddenLayer = [forecastingAgent getHiddenLayer];
  hiddenLayerTransFuncDerivatives = [forecastingAgent
                                     getHiddenLayerTransFuncDerivatives];
  outputLayer = [forecastingAgent getOutputLayer];
  who = [forecastingAgent getWho];
  dwho= [forecastingAgent getDwho];
  inputLayer  = [forecastingAgent getInputLayer];
  hiddenLayer = [forecastingAgent getHiddenLayer];
  hiddenLayerTransFuncDerivatives = [forecastingAgent
                                     getHiddenLayerTransFuncDerivatives];
  outputLayer = [forecastingAgent getOutputLayer];
  outputLayerTransFuncDerivatives = [forecastingAgent
                                     getOutputLayerTransFuncDerivatives];
  deltaOut = [forecastingAgent getDeltaOut];

  eps=[forecastingAgent getEps];
  alpha=[forecastingAgent getAlpha];
  usingRandomOrderInLearning=[forecastingAgent getRandomOrderInLearning];

  k=inputNodeNumber+outputNodeNumber; // k is introduced to simplify the code



// to build a vector of patterns and to randomize them
   if(NULL != p)free(p); // if p has never been used, the address stored in it
                         // (as set in createEnd) NULL
                         // we are deallocating and reallocating the p vector
                         // each time RuleMaker is invoked, otherwise we
                         // would have the problem of allocating a p vector
                         // with a static number of elements (may be,
                         // in future development of the code, this number
                         // could change from agent to agent ...)
   p= (int *) calloc(patternNumberInTrainingSet,sizeof(int));
   for (i=0;i<patternNumberInTrainingSet;i++)p[i]=i;

   // targets in each pattern
   for (i=0;i<patternNumberInTrainingSet;i++)
   for (j=0;j<outputNodeNumber;j++)
   [targetTrainingMatrix R: i C: j setFrom:
                        [forecastingTrainingSet R: i C: inputNodeNumber+j]];

   epochNumber=epochNumberInEachTrainingCycle;

// to start epochNumberInEachTrainingCycle (copied in epochNumber)
// training steps
   for (n=0; n<epochNumber;n++)
{

// to randomize pattern vector p
   for (i=patternNumberInTrainingSet; i>1; i--)
   {
   // iii randomly defined in the range 0 to i-1
   iii=[myUniformIntRand getIntegerWithMin: 0 withMax: i-1];
   temp=p[iii];
   p[iii]=p[i-1];
   p[i-1]=temp;    // vector p is so filled in random order, reordering
   }               // its contents from the last to the first


   for (I=0;I<patternNumberInTrainingSet;I++)
 {
   if(usingRandomOrderInLearning==1) i=p[I];
   else                              i=I;


   // building inputLayer, i.e. the (1,x')' vector in
   // y=f(B (1, f(A (1,x')')' )' )
   [inputLayer P: 0 setFrom: 1.]; // the bias, i.e. an artificial input
                                  // always set to 1, whose multiplying
                                  // parameters in A will operate as constant
                                  // values within the summations tranformed
                                  // by f(.)
   for (j=0;j<inputNodeNumber;j++)
       [inputLayer P: j+1 setFrom: [forecastingTrainingSet R: i C: j]];

   // to build hiddenLayer, i.e. to do the A (1,x')' operation
   // wih (input to hidden) is the A matrix
   [matrixMult m1: wih m2: inputLayer  to1: hiddenLayer];

   // to calculate derivatives, applying
   // d/dsi f(A (1,x')') where si is an element of the s=A (1,x')' vector
   [vectorTransFunc from1: hiddenLayer to1Derivative:
   //                                  ^^
   //                               to1 indicates a +1 displacement
   //                               in the resulting vector
        hiddenLayerTransFuncDerivatives];
   // these derivatives will be useful in the backpropagation calculation

   // to apply the transformation function, i.e. to calculate
   // f(A (1,x')')
   [vectorTransFunc from1: hiddenLayer to1: hiddenLayer];
   // nb: do not shift this line before the previous one; this transformation
   // changes the values of the hiddenLayer vector

   // hidden bias
   [hiddenLayer P: 0 setFrom: 1.];// the bias, i.e. an artificial hidden node
                                  // always set to 1, whose multiplying
                                  // parameters in B will operate as constant
                                  // values within the summations tranformed
                                  // by f(.)

   // to build the output layer, i.e. to do the B (1, f(.)' )' operation
   // who (hidden to output) is the B matrix
   [matrixMult m1: who m2: hiddenLayer to: outputLayer];

   // to calculate derivatives, applying
   // d/dzi f(B (1, f(.)' )' ) where zi is an element of
   // the z=B (1, f(.)' )' vector
   [vectorTransFunc from: outputLayer toDerivative:
                                          outputLayerTransFuncDerivatives];
   // these derivatives will be useful in the backpropagation calculation

   // to apply the transformation function, i.e. to calculate
   // f(B (1, f(.)' )' )
   [vectorTransFunc from: outputLayer to: outputLayer];
   // nb: do not shift this line before the previous one; this transformation
   // changes the values of the hiddenLayer vector

   // outputs in each pattern
   for (j=0;j<outputNodeNumber;j++)
   [outputTrainingMatrix R: i C: j setFrom: [outputLayer P: j]];


  // Error: we use here 1/2 to simplify derivatives of a widely applied
  // measure of quadratic error;
  // as a consequence, the usual learning rate eps=0.6 is actually
  // equivalent to a 0.3 correction (due to the error definition)

  // E = 1/2 SUM (t-o)^2
  // where SUM is the summation operator, operating over j, the output index
  //       t (target) is the actual value to be approximated
  //       o is an element of y=f(B (1, f(A (1,x')')' )' ),
  //        the vectors containing the outputs of the neural network function

  // we are now calcutating the derivatives of E with respect to each
  // parameter of the function (i.e. each element of A and B)
  // dE/dw where w is an element of who (B) or wih (A)

  // dE/dwho = -(t-o) Df(B (1, f(.)' )' ) H
  // where H is a hidden node value, i.e. an element in the
  // f(A (1,x')') vector

  // we define deltaOut=(t-o) Df(B (1, f(.)' )' )

  // deltaOut determination
  for (j=0;j<outputNodeNumber;j++)
  [deltaOut P: j setFrom: ([targetTrainingMatrix R: i C: j]-
                         [outputLayer P: j])*
                         [outputLayerTransFuncDerivatives P: j] ];

  // dwho (correction of who on the basis of -dE/dwho, with eps and the
  // momentum alpha)
  for (row=0;row<outputNodeNumber; row++)
  for (col=0;col<hiddenNodeNumber1;col++)
  [dwho R: row C: col setFrom: eps*[deltaOut P: row]* [hiddenLayer P: col]
                        + alpha*[dwho R: row C: col]];

  // dE/dwih = SUM[-(t-o) Df(B (1, f(.)' )' )] Df(A (1,x')') wih I
  // where I is an input value, i.e. an element in the (1,x')' vector

  // dwih (correction of wih on the basis of -dE/dwih, with eps and the
  // momentum alpha)
  for (row=0;row<hiddenNodeNumber; row++)
  {
        sum=0;
        for (colOut=0;colOut<outputNodeNumber;colOut++)
        sum+=[deltaOut P: colOut]*[who R: colOut C: row+1];

        delta=sum * [hiddenLayerTransFuncDerivatives P: row+1];

  for (col=0;col<inputNodeNumber1;col++)
  [dwih R: row C: col setFrom: eps*delta*[inputLayer P: col]
                        + alpha*[dwih R: row C: col]];
  }

  // who and wih correction
  // 'setFromP' operates as '+='
  for (row=0;row<outputNodeNumber; row++)
  for (col=0;col<hiddenNodeNumber1;col++)
  [who R: row C: col setFromP: [dwho R: row C: col]];

  for (row=0;row<hiddenNodeNumber; row++)
  for (col=0;col<inputNodeNumber1; col++)
  [wih R: row C: col setFromP: [dwih R: row C: col]];

 }


  if (n==epochNumber-1) // last epoch
  {
  // errors at the end of each learning session-cycle (after 
  // epochNumberInEachTrainingCycle computations)

  // here the solution is completely different from that adopted in the
  // bp-ct scheme, where the evaluation is at n==0 (see in bp-ct package)

  backPropagationError=0;
  proportionalError=0;

   for (i=0;i<patternNumberInTrainingSet;i++)
   {

   // conventional back propagation error (sum of all the output units in
   // all the patterns, i.e. epoch error, divided by the no. of patterns)
   // the error is also conventionally divided by 2

   for (j=0;j<outputNodeNumber;j++)
      backPropagationError+=pow([outputTrainingMatrix R: i C: j]-
                                [targetTrainingMatrix R: i C: j],2)/
                                (2*patternNumberInTrainingSet);

  // proportional error (arithmetic mean of all the output units in
   // all the patterns, i.e. epoch error)

   // this error is the ratio obtained dividing the abs value of
   // the difference of each target minus each output by one half of the
   // difference of the internal max value {i.e. [minmax R:3 C:j]} minus
   // the internal min value {i.e. [minmax R:2 C:j]} (the metrics is here
   // that of the internal values; anyway, when calculating a ratio, the
   // metrics is not relevant)

   // this is a raw measure of the error as a proportion
   // of the range (divided by 2) of validity the output values

   // we have max error=2; min error=0

   for (j=0;j<outputNodeNumber;j++){
       proportionalError+=
          ( fabs([outputTrainingMatrix R: i C: j]
               - [targetTrainingMatrix R: i C: j]) /
               (([minmax R:j+inputNodeNumber C:3] -
                 [minmax R:j+inputNodeNumber C:2])/2.)
          ) / (outputNodeNumber*patternNumberInTrainingSet);  }
                   // the last quotient calculates the mean upon
                   // all patterns and output units


  }}

}

   return self;
}

-(float) getBackPropagationErrorInTrainingSet
{
  return backPropagationError;
}

-(float) getProportionalErrorInTrainingSet
{
  return proportionalError;
}

@end

