// StopLossAgent.h 

// StopLossAgent inherits from BasicSumAgent and operates in random way,
// like a random agent (also checking if the price is sliding below the floor price,
// where aa buying behavior prevails)

// After that, its behavior can be modified by a "stop loss"
// decision, operating in two possible ways 
//
//(i) without memory (the short or long position, if any, of the agent is not
//    relevant) / NB the stop loss decision is set after that of operating
//                below the floor price and so can supersede that decision
//
//    if the current price, i.e. the last executedPrice (at day t) is
//       >= to (mean price at day t-stopLossInterval) *(1+maxLossRate)
//       <= to (mean price at day t-stopLossInterval) *(1-maxLossRate)
//       and checkingIfShortOrLong=0,
//    the agent buys/sells, at the current price, a quantity of shares between 1 
//    and maxOrderQuantity
//
//(ii) with memory (the short or long position of the agent is taken in account; if
//     the agent has no position, doesn't operate here)
//              / NB the stop loss decision is set after that of operating
//                below the floor price and so can supersede that decision
//
//    if the current price, i.e. the last executedPrice (at day t) is
//       >= to (mean price at day t-stopLossInterval) *(1+maxLossRate)
//       <= to (mean price at day t-stopLossInterval) *(1-maxLossRate)
//       and checkingIfShortOrLong=1,
//    the agent 
//             buys  if short (shareQuantity<0)  /
//             sells if long  (shareQuantity>0)
//    at the current price, a quantity of shares between 1 
//    and maxOrderQuantity  (regardless its shareQuantity value)

// anyway, in case of price sliding below the floor price, a buying behavior
// prevails

#import "BasicSumAgent.h"
#import "StopLossRuleMaster.h"

@interface StopLossAgent: BasicSumAgent
{
  int stopLossInterval, checkingIfShortOrLong;
  float maxLossRate;

  StopLossRuleMaster * ruleMaster;

}

- setRuleMaster: r;

- setStopLossInterval: (int) i;
- setMaxLossRate: (float) r andCheckingIfShortOrLong: (int) sl;

- act0; // acting - placing buy or sell orders - before opening
- act1;  // placing buy or sell orders

@end



